@@ -18,7 +18,18 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
 
 **************************************************************************** */
 #ifndef KBABEL_H
@@ -117,6 +128,11 @@ protected:
      */
     virtual void dragEnterEvent(QDragEnterEvent *event);
     virtual void dropEvent(QDropEvent *event);
+    
+    /**
+     * Overrriden virtual for wheel event handling to forward to KBabelView
+     */
+    virtual void wheelEvent(QWheelEvent *e);
 
     /**
      * This function is called when it is time for the app to save its
@@ -152,8 +168,10 @@ private slots:
     void projectOpen();
     void projectClose();
     void projectConfigure();
+    void projectOpenRecent(const KURL& url);
 
     void addToRecentFiles(KURL url);
+    void addToRecentProjects(KURL url);
 
     void optionsShowStatusbar(bool);
     void optionsEditToolbars();
@@ -283,6 +301,7 @@ private:
     KAction* a_nextUntrans;
     
     KRecentFilesAction* a_recent;
+    KRecentFilesAction* a_recentprojects;
 
     DictionaryMenu *dictMenu;
     DictionaryMenu *selectionDictMenu;
