@@ -3,6 +3,8 @@
 
   Copyright (C) 2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
+		2003 by Stanislav Visnovsky
+			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,11 +18,22 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+	    
 **************************************************************************** */
 #include <klocale.h>
-#include <kapp.h>
+#include <kapplication.h>
 #include <kcmdlineargs.h>
 #include <kstaticdeleter.h>
 #include <kdebug.h>
@@ -28,20 +41,25 @@
 #include <kinstance.h>
 #include <kio/netaccess.h>
 
+#include <qregexp.h>
 #include <qtextstream.h>
 #include <qtimer.h>
 
+#include "resources.h"
 #include "compendiumdata.h"
 #include "pocompendium.h"
 #include "preferenceswidget.h"
 #include "pc_factory.h"
 
+#include <catalog.h>
 #include <catalogitem.h>
 
 // ngram length and minimal matching for fuzzy search
 #define NGRAM_LEN 3
 #define LIM_NGRAM 50  
 
+using namespace KBabel;
+
 static KStaticDeleter< QDict<CompendiumData> > compDataDeleter;
 QDict<CompendiumData> *PoCompendium::_compDict = 0;
 
@@ -49,7 +67,6 @@ PoCompendium::PoCompendium(QObject *parent, const char *name)
         : SearchEngine(parent, name)
 {
     prefWidget=0;
-    conf=0;
     data=0;
     error=false;
     stop=false;
@@ -96,7 +113,7 @@ bool PoCompendium::isSearching() const
 }
 
 
-void PoCompendium::saveSettings(KConfigBase *config)
+void PoCompendium::saveSettings(KConfigBase *config )
 {
     if(autoUpdate && prefWidget && prefWidget->settingsChanged())
     {
@@ -141,14 +158,12 @@ void PoCompendium::readSettings(KConfigBase *config)
 
             
     restoreSettings();
-
-    conf=config;
-    confGroup=config->group();
 }
 
 PrefWidget *PoCompendium::preferencesWidget(QWidget *parent)
 {
-    prefWidget = new PreferencesWidget(parent,"pocompendium_prefwidget");
+    prefWidget = new CompendiumPreferencesWidget(parent,"pocompendium_prefwidget");
+    kdDebug(KBABEL_SEARCH) << "PreferencesWidget is " << prefWidget << endl;
     connect(prefWidget, SIGNAL(applySettings()), this, SLOT(applySettings()));
     connect(prefWidget, SIGNAL(restoreSettings())
                     , this, SLOT(restoreSettings()));
@@ -179,48 +194,8 @@ QString PoCompendium::lastError()
     return errorMsg;
 }
 
-
-bool PoCompendium::startSearch(QString text)
+bool PoCompendium::searchExact(const QString& text, uint pluralForm, QPtrList<SearchResult>& results, QValueList<int>& foundIndices, QValueList<int>& )
 {
-    if(autoUpdate && prefWidget && prefWidget->settingsChanged())
-    {
-        applySettings();
-    }
- 
-    if(isSearching())
-        return false;
-
-    clearResults();
-    stop = false;
-    active = true;
-
-    if(!initialized)
-    {
-        if(loadTimer->isActive())
-            loadTimer->stop();
-
-        slotLoadCompendium();
-    }
-    
-    if(error || !data)
-    {
-        active = false;
-        return false;
-    }
-
-    if(data->active())
-    {
-        active = false;
-        return true;
-    }
-   
-    emit started();
-
-
-    QValueList<int> foundIndices;
-    QValueList<int> checkedIndices;
-    uint checkCounter=0;
-    
     const int *index = data->exactDict(text);
     if(index)
     {
@@ -229,7 +204,8 @@ bool PoCompendium::startSearch(QString text)
         SearchResult *result = new SearchResult;
         result->requested = text;
         result->found = data->catalog()->msgid(*index);
-        result->translation = data->catalog()->msgstr(*index);
+#warning	kdWarning() << "PoCompendium can't return plural form translation" << endl;
+        result->translation = data->catalog()->msgstr(*index).first();
         result->score = 100;
 
         TranslationInfo *info = new TranslationInfo;
@@ -238,24 +214,19 @@ bool PoCompendium::startSearch(QString text)
         info->description = data->catalog()->comment(*index);
         result->descriptions.append(info);
 
-        results.append(result);
-
-        emit numberOfResultsChanged(results.count());
-        emit resultFound(result);
+	addResult(result, results);
+	return true;
     }
-
-    QString searchStr=CompendiumData::simplify(text);
     
-           
-    if(!caseSensitive)
-    {
-         searchStr = searchStr.lower();
-    }
+    return false;
 
-    QString temp = searchStr;
-    temp = temp.lower();
+}
 
-    const QValueList<int> *indexList = data->allDict(temp);
+bool PoCompendium::searchCaseInsensitive(const QString& text, uint pluralForm, QPtrList<SearchResult>& results, QValueList<int>& foundIndices, QValueList<int>& )
+{
+    QString searchStr = text.lower();
+    
+    const QValueList<int> *indexList = data->allDict(text.lower());
     if(indexList)
     {
         QValueList<int>::ConstIterator it;
@@ -272,7 +243,7 @@ bool PoCompendium::startSearch(QString text)
             }
 
 
-            QString origStr = data->catalog()->msgid(*it);
+            QString origStr = data->catalog()->msgid(*it).first();
             origStr = CompendiumData::simplify(origStr);
         
            
@@ -288,20 +259,77 @@ bool PoCompendium::startSearch(QString text)
                 SearchResult *result = new SearchResult;
                 result->requested = text;
                 result->found = data->catalog()->msgid(*it);
-                result->translation = data->catalog()->msgstr(*it);
-                result->score = score(result->requested,result->found);
+                result->translation = *(data->catalog()->msgstr(*it).at(pluralForm));
+                result->score = score(result->requested,*(result->found.at(pluralForm)));
 
                 TranslationInfo *info = new TranslationInfo;
                 info->location = directory(realURL,0);
                 info->translator = catalogInfo.lastTranslator;
                 info->description = data->catalog()->comment(*it);
                 result->descriptions.append(info);
- 
-                addResult(result);
+		
+		addResult(result, results);
+		return true;
             }
         }
     }
- 
+    return false;
+}
+
+bool PoCompendium::searchTextOnly(const QString& text, uint pluralForm, QPtrList<SearchResult>& results, QValueList<int>& foundIndices, QValueList<int>& )
+{
+    QString searchStr = text.lower();
+    QString t = text;
+    t.remove( " " );
+
+    const QValueList<int> *indexList = data->textonlyDict(t.lower());
+    if(indexList)
+    {
+        QValueList<int>::ConstIterator it;
+        for( it = indexList->begin(); it != indexList->end(); ++it )
+        {
+            if(foundIndices.contains(*it))
+            {
+                continue;
+            }
+
+            if(ignoreFuzzy && data->catalog()->isFuzzy(*it))
+            {
+                continue;
+            }
+
+
+            QString origStr = data->catalog()->msgid(*it).first();
+            origStr = CompendiumData::simplify(origStr);
+        
+           
+            foundIndices.append(*it);
+
+            SearchResult *result = new SearchResult;
+            result->requested = text;
+            result->found = data->catalog()->msgid(*it).first();
+#warning	    kdWarning() << "PoCompendium can't return plural form translation" << endl;
+            result->translation = data->catalog()->msgstr(*it).first();
+            result->score = score(result->requested,*(result->found.at(pluralForm)));
+
+            TranslationInfo *info = new TranslationInfo;
+            info->location = directory(realURL,0);
+            info->translator = catalogInfo.lastTranslator;
+            info->description = data->catalog()->comment(*it);
+            result->descriptions.append(info);
+		
+	    addResult(result, results);
+	    return true;
+        }
+    }
+    return false;
+}
+
+bool PoCompendium::searchWords(const QString& searchStr, uint pluralForm, QPtrList<SearchResult>& results, QValueList<int>& foundIndices, QValueList<int>& checkedIndices )
+{
+    uint checkCounter = 0;
+    bool foundResults = false;
+    
     QStringList wList = CompendiumData::wordList(searchStr);
     for ( QStringList::Iterator wit = wList.begin()
                ; wit != wList.end(); ++wit ) 
@@ -309,7 +337,7 @@ bool PoCompendium::startSearch(QString text)
         if(stop)
             break;
 
-        indexList = data->wordDict((*wit).lower());
+        const QValueList<int> *indexList = data->wordDict((*wit).lower());
         if(indexList)
         {
             QValueList<int>::ConstIterator it;
@@ -338,12 +366,12 @@ bool PoCompendium::startSearch(QString text)
  
                 if( (100*(checkCounter+1))%catalogInfo.total < 100)
                 {
-                    emit progress( (100*(checkCounter+1))/catalogInfo.total);
+                    emit progress( (50*checkCounter+1)/catalogInfo.total);
                 }
             
-                kapp->processEvents();
+                kapp->processEvents(100);
 
-                QString origStr = data->catalog()->msgid(*it);
+                QString origStr = data->catalog()->msgid(*it).first();
                 origStr = CompendiumData::simplify(origStr);
 
            
@@ -385,7 +413,7 @@ bool PoCompendium::startSearch(QString text)
                 // if there is an string which just contains ignored characters,
                 // continue
                 if(!found && origStr.isEmpty() || 
-                    (origStr.length() == 1 && text.length() > 1))
+                    (origStr.length() == 1 && searchStr.length() > 1))
                 {
                     continue;
                 }
@@ -396,7 +424,7 @@ bool PoCompendium::startSearch(QString text)
                     QString s=maskString(searchStr);
                     QRegExp searchReg(s);
 
-                    if(searchReg.match(origStr) >= 0)
+                    if(searchReg.search(origStr) >= 0)
                         found=true;
                 }
             
@@ -405,7 +433,7 @@ bool PoCompendium::startSearch(QString text)
                     QString s=maskString(origStr);
                     QRegExp reg(s);
 
-                    if(reg.match(searchStr) >= 0)
+                    if(reg.search(searchStr) >= 0)
                     {
                         found = true;
                     }
@@ -420,7 +448,7 @@ bool PoCompendium::startSearch(QString text)
                         QString s=maskString(*wit2);
                         QRegExp reg(s);
 
-                        if(reg.match(origStr) >= 0)
+                        if(reg.search(origStr) >= 0)
                         {
                             found = true;
                         }
@@ -432,10 +460,11 @@ bool PoCompendium::startSearch(QString text)
                     foundIndices.append(*it);
 
                     SearchResult *result = new SearchResult;
-                    result->requested = text;
-                    result->found = data->catalog()->msgid(*it);
-                    result->translation = data->catalog()->msgstr(*it);
-                    result->score = score(result->requested,result->found);
+                    result->requested = searchStr; // FIXME:here should be the original text
+                    result->found = data->catalog()->msgid(*it).first();
+#warning		    kdWarning() << "PoCompendium can't return plural form translation" << endl;
+                    result->translation = data->catalog()->msgstr(*it).first();
+                    result->score = score(result->requested,*(result->found.at(pluralForm)));
 
                     TranslationInfo *info = new TranslationInfo;
                     info->location = directory(realURL,0);
@@ -443,164 +472,236 @@ bool PoCompendium::startSearch(QString text)
                     info->description = data->catalog()->comment(*it);
                     result->descriptions.append(info);
  
-                    addResult(result);
+                    addResult(result, results);
+		    
+		    foundResults = true;
                 }
             }
         }
     }
+    
+    return foundResults;
+}
+
+bool PoCompendium::searchNGram(const QString& searchStr, uint pluralForm, QPtrList<SearchResult>& results, QValueList<int>& foundIndices, QValueList<int>& checkedIndices )
+{
+    uint checkCounter = 0;
+    bool foundResults = false;
+    
+    QRegExp searchReg;
+    if(matchContains)
+    {
+        QString s=maskString(searchStr);
+        searchReg.setPattern(s);
+    }
+
+        
+    bool breakLoop=false;
+    int i=-1;
+    while(!breakLoop)    
+    {
+        if(stop)
+        {
+    	    breakLoop=true;
+            break;
+        }
+
+	i++;
+            
+        if(i >= catalogInfo.total)
+        {
+            breakLoop = true;
+            break;
+        }
+        
+        if( (100*(checkCounter+1))%catalogInfo.total < 100)
+        {
+            emit progress( 50+(50*(checkCounter+1))/catalogInfo.total);
+        }
+
+        if(checkedIndices.contains(i))
+        {
+            continue;
+        }
+
+        checkedIndices.append(i);
+        checkCounter++;
+  
+        if(ignoreFuzzy && data->catalog()->isFuzzy(i))
+        {
+    	     continue;
+        }
+        
+        if(foundIndices.contains(i))
+        {
+            continue;
+        }
+       
+        kapp->processEvents(100);
+
+        QString origStr = data->catalog()->msgid(i).first();
+        origStr = CompendiumData::simplify(origStr);
+            
+        if(!caseSensitive)
+        {
+            origStr = origStr.lower();
+        }
+
+ 
+        // if there is an string which just contains ignored characters,
+        // continue
+        if(origStr.isEmpty() || 
+                    (origStr.length() == 1 && searchStr.length() > 1))
+        {
+            continue;
+        }
+            
+        bool found = false;
+        if(matchContains && searchReg.search(origStr) >= 0)
+        {
+            found=true;
+        }
+
+        if(!found && matchIsContained)
+        {
+            QString s=maskString(origStr);
+            QRegExp reg(s);
+
+            if(reg.search(searchStr) >= 0)
+            {
+                found = true;
+            }
+        }
+
+        if(!found && matchWords)
+        {
+            QStringList list = CompendiumData::wordList(searchStr);
+                    
+            for ( QStringList::Iterator wit2 = list.begin()
+                   ; wit2 != list.end(); ++wit2 ) 
+            {
+                QString s=maskString(*wit2);
+
+                if(wholeWords)
+                {
+                    origStr = " "+origStr+" ";
+                    s=" "+s+" ";
+                }
+                QRegExp reg(s);
+                    
+                if(reg.search(origStr) >= 0)
+                {
+                    found = true;
+                }
+            }
+        }
+
+	if(!found && matchNGram)
+	{
+		// to get more results one could
+		// interchange searchStr and origStr when
+		// the latter is shorter
+
+	    found = ( ngramMatch(searchStr,origStr,NGRAM_LEN) 
+			  > LIM_NGRAM );
+	}
+       
+        if(found)
+        {
+            foundIndices.append(i);
+
+            SearchResult *result = new SearchResult;
+            result->requested = searchStr;
+            result->found = data->catalog()->msgid(i).first();
+            result->translation = data->catalog()->msgstr(i).first();
+#warning	    kdWarning() << "PoCompendium can't return plural form translation" << endl;
+            result->score = score(result->requested,*(result->found.at(pluralForm)));
+
+            TranslationInfo *info = new TranslationInfo;
+            info->location = directory(realURL,0);
+            info->translator = catalogInfo.lastTranslator;
+            info->description = data->catalog()->comment(i);
+            result->descriptions.append(info);
+    
+            addResult(result, results);
+	    
+	    foundResults = true;
+        }
+    }
+    
+    return foundResults;
+}
+
+bool PoCompendium::startSearch(const QString& text, uint pluralForm, const SearchFilter* filter)
+{
+    if(autoUpdate && prefWidget && prefWidget->settingsChanged())
+    {
+        applySettings();
+    }
+ 
+    if(isSearching())
+        return false;
+
+    clearResults();
+    stop = false;
+    active = true;
+
+    if(!initialized)
+    {
+        if(loadTimer->isActive())
+            loadTimer->stop();
+
+        slotLoadCompendium();
+    }
+    
+    if(error || !data)
+    {
+        active = false;
+        return false;
+    }
+
+    if(data->active())
+    {
+        active = false;
+        return true;
+    }
+   
+    emit started();
+
+
+    QValueList<int> foundIndices;
+    QValueList<int> checkedIndices;
+    
+    // first, exact search
+    searchExact(text, pluralForm, results, foundIndices, checkedIndices);
+    
+    QString searchStr=CompendiumData::simplify(text);
+    
+           
+    if(!caseSensitive)
+    {
+         searchStr = searchStr.lower();
+
+	// do case insensite search if necessary
+	searchCaseInsensitive(searchStr, pluralForm, results, foundIndices, checkedIndices);
+    }
+    
+    // search without whitespace
+    searchTextOnly(searchStr, pluralForm, results, foundIndices, checkedIndices);
+    
+    // now, search based on words (contains, is contained, etc)
+    searchWords(searchStr, pluralForm, results, foundIndices, checkedIndices);
  
     
     if( matchNGram || 
 	(!wholeWords && (matchContains || matchIsContained || matchWords))
 	)
-      {
-        QRegExp searchReg;
-        if(matchContains)
-        {
-            QString s=maskString(searchStr);
-            searchReg.setPattern(s);
-        }
-
-        
-        bool breakLoop=false;
-        int i=-1;
-        while(!breakLoop)    
-        {
-            if(stop)
-            {
-                breakLoop=true;
-                break;
-            }
-
-            i++;
-            
-            if(checkedIndices.contains(i))
-            {
-                continue;
-            }
-
-            checkedIndices.append(i);
-            checkCounter++;
-  
-            if(ignoreFuzzy && data->catalog()->isFuzzy(i))
-            {
-                 continue;
-            }
-        
-            if(foundIndices.contains(i))
-            {
-                 continue;
-            }
-       
-            kapp->processEvents();
-
-            if(i >= catalogInfo.total)
-            {
-                 breakLoop = true;
-                 break;
-            }
-        
-            if( (100*(checkCounter+1))%catalogInfo.total < 100)
-            {
-                emit progress( (100*(checkCounter+1))/catalogInfo.total);
-            }
-
-             QString origStr = data->catalog()->msgid(i);
-             origStr = CompendiumData::simplify(origStr);
-            
-             if(!caseSensitive)
-             {
-                 origStr = origStr.lower();
-             }
-
- 
-            // if there is an string which just contains ignored characters,
-            // continue
-            if(origStr.isEmpty() || 
-                    (origStr.length() == 1 && text.length() > 1))
-            {
-                continue;
-            }
-            
-            bool found = false;
-            if(matchContains && searchReg.match(origStr) >= 0)
-            {
-                found=true;
-            }
-
-                    
-            if(!found && matchIsContained)
-            {
-                QString s=maskString(origStr);
-                QRegExp reg(s);
-
-                if(reg.match(searchStr) >= 0)
-                {
-                    found = true;
-                }
-            }
-            if(!found && matchWords)
-            {
-                QStringList list = CompendiumData::wordList(searchStr);
-                    
-                for ( QStringList::Iterator wit2 = list.begin()
-                   ; wit2 != list.end(); ++wit2 ) 
-                {
-                    QString s=maskString(*wit2);
-
-                    if(wholeWords)
-                    {
-                        origStr = " "+origStr+" ";
-                        s=" "+s+" ";
-                    }
-                    QRegExp reg(s);
-                    
-                    if(reg.match(origStr) >= 0)
-                    {
-                        found = true;
-                    }
-                }
-             }
-
-	    if(!found && matchNGram)
-	      {
-		// to get more results one could
-		// interchange searchStr and origStr when
-		// the latter is shorter
-
-		found = ( ngramMatch(searchStr,origStr,NGRAM_LEN) 
-			  > LIM_NGRAM );
-	      }
-       
-            if(found)
-            {
-                 foundIndices.append(i);
-
-                 SearchResult *result = new SearchResult;
-                 result->requested = text;
-                 result->found = data->catalog()->msgid(i);
-                 result->translation = data->catalog()->msgstr(i);
-                 result->score = score(result->requested,result->found);
-
-                 TranslationInfo *info = new TranslationInfo;
-                 info->location = directory(realURL,0);
-                 info->translator = catalogInfo.lastTranslator;
-                 info->description = data->catalog()->comment(i);
-                 result->descriptions.append(info);
-    
-                 addResult(result);
-             }
-        }
+    {
+	// search based on ngrams
+	searchNGram(searchStr, pluralForm, results, foundIndices, checkedIndices);
     } 
     
-    if( (100*(checkCounter+1))/catalogInfo.total < 100 && !stop)
-    {
-        for(int i=(100*(checkCounter+1))/catalogInfo.total; i<=100; i++)
-        {
-            emit progress(i);
-        }
-    }
-    
+    emit progress(100);
 
     active = false;
     stop = false;
@@ -648,14 +749,6 @@ void PoCompendium::applySettings()
         needLoading = true;
     }
 
-
-    if(conf)
-    {
-        KConfigGroupSaver gs(conf,confGroup);
-        saveSettings(conf);
-    }
-
-    
     if(needLoading)
     {
         loadCompendium();
@@ -706,7 +799,7 @@ void PoCompendium::slotLoadCompendium()
    
     if(path.contains("@LANG@"))
     {
-        path.replace(QRegExp("@LANG@"),langCode);
+        path.replace("@LANG@",langCode);
     }
     KURL u=KCmdLineArgs::makeURL(path.local8Bit());
     realURL = u.url();
@@ -773,37 +866,39 @@ void PoCompendium::recheckData()
 
 QString PoCompendium::maskString(QString s)
 {
-    s.replace(QRegExp("\\\\"),"\\\\");
-    s.replace(QRegExp("\\?"),"\\?");
-    s.replace(QRegExp("\\["),"\\[");
-    s.replace(QRegExp("\\."),"\\.");
-    s.replace(QRegExp("\\*"),"\\*");
-    s.replace(QRegExp("\\+"),"\\+");
-    s.replace(QRegExp("\\^"),"\\^");
-    s.replace(QRegExp("\\$"),"\\$");
-    s.replace(QRegExp("\\("),"\\(");
-    s.replace(QRegExp("\\)"),"\\)");
-    s.replace(QRegExp("\\{"),"\\{");
-    s.replace(QRegExp("\\}"),"\\}");
-    s.replace(QRegExp("\\|"),"\\|");
+    s.replace("\\","\\\\");
+    s.replace("\?","\\?");
+    s.replace("[","\\[");
+    s.replace(".","\\.");
+    s.replace("*","\\*");
+    s.replace("+","\\+");
+    s.replace("^","\\^");
+    s.replace("$","\\$");
+    s.replace("(","\\(");
+    s.replace(")","\\)");
+    s.replace("{","\\{");
+    s.replace("}","\\}");
+    s.replace("|","\\|");
     
     return s;
 }
 
-void PoCompendium::addResult(SearchResult *result)
+void PoCompendium::addResult(SearchResult *result, QPtrList<SearchResult>& res)
 {
-    if(results.last() && results.last()->score >= result->score)
+    if(res.last() && res.last()->score >= result->score)
     {
-        results.append(result);
+        res.append(result);
     }
     else
     {
         SearchResult *sr;
-        for(sr = results.first(); sr != 0; sr=results.next())
+        for(sr = res.first(); sr != 0; sr=res.next())
         {
             if(sr->score < result->score)
             {
-                results.insert(results.at(),result);
+		int pos = res.at();
+		if( pos < 0 ) pos = 0;
+                res.insert( pos,result);
                 emit resultsReordered();
                 break;
             }
@@ -811,16 +906,16 @@ void PoCompendium::addResult(SearchResult *result)
 
         if(!sr)
         {
-             results.append(result);
+             res.append(result);
         }
     }
 
-    emit numberOfResultsChanged(results.count());
+    emit numberOfResultsChanged(res.count());
     emit resultFound(result);
 }
 
 
-void PoCompendium::setLanguageCode(QString lang)
+void PoCompendium::setLanguageCode(const QString& lang)
 {   
     if(initialized && url.contains("@LANG@") && lang!=langCode 
             && !loadTimer->isActive() )
@@ -831,7 +926,7 @@ void PoCompendium::setLanguageCode(QString lang)
     langCode=lang;
 }
 
-QString PoCompendium::translate(const QString text)
+QString PoCompendium::translate(const QString& text, uint pluralForm)
 {
     if(!initialized)
     {
@@ -851,13 +946,14 @@ QString PoCompendium::translate(const QString text)
 
     if(index)
     {
-        return data->catalog()->msgstr(*index);
+#warning	kdWarning() << "PoCompendium can't return plural form translation" << endl;
+        return data->catalog()->msgstr(*index).first();
     }
    
     return QString::null;
 }
 
-QString PoCompendium::fuzzyTranslation(const QString text, int &score)
+QString PoCompendium::fuzzyTranslation(const QString& text, int &score, const uint pluralForm)
 {
   if(!initialized)
     {
@@ -907,8 +1003,8 @@ QString PoCompendium::fuzzyTranslation(const QString text, int &score)
 	  emit progress( (100*(i+1))/total);
 	}
 
-      // get a message from the catalog
-      QString origStr = data->catalog()->msgid(i);
+      // get a message from the catalog FIXME: plurals
+      QString origStr = data->catalog()->msgid(i).first();
       origStr = CompendiumData::simplify(origStr);
 
       // don't match too long strings for short search string
@@ -929,19 +1025,136 @@ QString PoCompendium::fuzzyTranslation(const QString text, int &score)
 
   if (best_match > LIM_NGRAM) {
     score = best_match;
-    return data->catalog()->msgstr(best_matching);
+#warning    kdWarning() << "PoCompendium can't return plural form translation" << endl;
+    return data->catalog()->msgstr(best_matching).first();
   }
  
   return QString::null;
 }
 
 
+QString PoCompendium::searchTranslation(const QString& text, int &sc, const uint pluralForm)
+{
+    if(autoUpdate && prefWidget && prefWidget->settingsChanged())
+    {
+        applySettings();
+    }
+ 
+    if(isSearching())
+        return QString::null;
+
+    clearResults();
+    stop = false;
+    active = true;
+
+    if(!initialized)
+    {
+        if(loadTimer->isActive())
+            loadTimer->stop();
+
+        slotLoadCompendium();
+    }
+    
+    if(error || !data)
+    {
+        active = false;
+        return QString::null;
+    }
+
+    if(data->active())
+    {
+        active = false;
+        return QString::null;
+    }
+   
+    emit started();
+
+    QPtrList<SearchResult> res;
+    
+    QValueList<int> foundIndices;
+    QValueList<int> checkedIndices;
+    
+    // first, exact search
+    if( searchExact(text, pluralForm, res, foundIndices, checkedIndices) )
+    {
+	active = false;
+	stop = false;
+	emit finished();
+
+	// found, this is the best
+	sc = res.first()->score;
+	return res.first()->translation;
+    }
+    
+    QString searchStr=CompendiumData::simplify(text);
+    
+           
+    if(!caseSensitive)
+    {
+        searchStr = searchStr.lower();
+
+	// do case insensite search if necessary
+	if( searchCaseInsensitive(searchStr, pluralForm, res, foundIndices, checkedIndices) )
+	{
+	    active = false;
+	    stop = false;
+	    emit finished();
+	    
+	    // found, return this one
+	    sc = res.first()->score;
+	    return res.first()->translation;
+	}
+    }
+    
+    // search without whitespace
+    QString s = searchStr;
+    s.remove( ' ' );
+    if( searchTextOnly(s, pluralForm, res, foundIndices, checkedIndices) )
+    {
+	active = false;
+	stop = false;
+	emit finished();
+
+	// found, return this one
+	sc = res.first()->score;
+	return res.first()->translation;
+    }
+    
+    // now, search based on words (contains, is contained, etc)
+    searchWords(searchStr, pluralForm, res, foundIndices, checkedIndices);
+    
+    if( matchNGram || 
+	(!wholeWords && (matchContains || matchIsContained || matchWords))
+	)
+    {
+	// search based on ngrams
+	searchNGram(searchStr, pluralForm, res, foundIndices, checkedIndices);
+    }
+    
+    active = false;
+    stop = false;
+
+    // now, pick up the best one from not exact translations
+    if( res.count() > 0 ) {
+
+	emit finished();
+
+	sc = res.first()->score;
+	return res.first()->translation;
+    }
+    
+    sc = 0;
+    
+    return QString::null;
+}
+
+
 void PoCompendium::unregisterData()
 {
     if(data)
     { 
-        disconnect(data, SIGNAL(progressStarts(const QString)), this
-            , SIGNAL(progressStarts(const QString)));
+        disconnect(data, SIGNAL(progressStarts(const QString&)), this
+            , SIGNAL(progressStarts(const QString&)));
         disconnect(data, SIGNAL(progressEnds()), this , SIGNAL(progressEnds()));
         disconnect(data, SIGNAL(progress(int)), this , SIGNAL(progress(int)));
 
@@ -979,11 +1192,12 @@ void PoCompendium::registerData()
     
     if(data->active())
     {
-        emit progressStarts(i18n("Loading po compendium"));
+        emit progressStarts(i18n("Loading PO compendium"));
     }
     
-    connect(data, SIGNAL(progressStarts(const QString)), this
-            , SIGNAL(progressStarts(const QString)));
+    connect(data, SIGNAL(
+    progressStarts(const QString&)), this
+            , SIGNAL(progressStarts(const QString&)));
     connect(data, SIGNAL(progressEnds()), this , SIGNAL(progressEnds()));
     connect(data, SIGNAL(progress(int)), this , SIGNAL(progress(int)));
 }
