@@ -4,6 +4,10 @@ using namespace std;
 #include <fstream>
 #include "GettextLexer.hpp"
 
+#include <qregexp.h>
+#include <qdatetime.h>
+#include <qfileinfo.h>
+
 int main(int argc, char **argv)
 {
     if ( argc != 2 && argc != 4 ) {
@@ -38,12 +42,33 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    bool is_desktop = filename.find( "desktop_") >= 0;
+    const bool is_desktop = filename.find( "desktop_") >= 0;
 
-    cout << "msgid \"\"\n";
-    cout << "msgstr \"\"\n";
-    cout << "\"Content-Type: text/plain; charset=utf-8\\n\"\n";
-    cout << "\"Plural-Forms: nplurals=1; plural=0;\\n\"\n";
+    // The header is the last item (due too the sorting)
+    MsgList::const_iterator header = --translated.end();
+    if ( ( header == translated.end() ) || ( ! ( *header ).msgid.isEmpty() ) )
+    {
+        cerr << "Cannot find correct header msgid\n";
+        cout << "\"Content-Type: text/plain; charset=utf-8\\n\"\n";
+        cout << "\"Plural-Forms: nplurals=1; plural=0;\\n\"\n";
+    }
+    else
+    {
+        QStringList headerLines = QStringList::split( "\\n", ( *header ).msgstr, false );
+        QFileInfo fi( QString::fromLocal8Bit( filename ) );
+        QString projectId( "Project-Id-Version: " ); 
+        projectId += fi.baseName( false );
+        headerLines.gres( QRegExp( "^Project-Id-Version:.*" ), projectId );
+        headerLines.gres( QRegExp( "^Last-Translator:.*" ), "Last-Translator: transxx program <null@kde.org>" );
+        headerLines.gres( QRegExp( "^Language-Team:.*" ), "Language-Team: Test Language <kde-i18n-doc@kde.org>" );
+        QString revisionDate ( "PO-Revision-Date: " );
+        const QDateTime dt = QDateTime::currentDateTime( Qt::UTC );
+        revisionDate += dt.toString( "yyyy-MM-dd hh:mm+0000" );
+        headerLines.gres( QRegExp( "^PO-Revision-Date:.*" ), revisionDate );
+        headerLines << "Plural-Forms: nplurals=1; plural=0;";
+        outputMsg ( "msgid", "" );
+        outputMsg ( "msgstr", escapePO( headerLines.join("\\n") + "\\n" ) );
+    }
     cout << "\n";
 
     for (MsgList::ConstIterator it = translated.begin();
@@ -83,9 +108,11 @@ int main(int argc, char **argv)
             else
                 msgstr = translation + msgid + translation;
 
-            if ( msgid.endsWith("\n" ) )
+            // Note: msgid has been modified, so we need to go back to the original version by the help of the iterator
+            // (Gettext is not aware of the KDE-specific handling, so it really wants a \n at start and at end in the msgstr if they were in the msgid )
+            if ( ( *it ).msgid.endsWith( "\\n" ) && ! ( *it ).msgid.endsWith( "\\\\n" ))
                 msgstr += "\n";
-            if ( msgid.startsWith( "\n" ))
+            if ( ( *it ).msgid.startsWith( "\\n" ) )
                 msgstr.prepend( "\n" );
 
 	    if ( msgid_plural.isEmpty() ) {
