@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2004
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *  copyright (C) 2004-2006                                                *
+ *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
  ***************************************************************************/
 
 // own header
@@ -26,6 +23,7 @@
 #include "umldoc.h"
 #include "uml.h"
 #include "listpopupmenu.h"
+#include "object_factory.h"
 
 ClassifierWidget::ClassifierWidget(UMLView * view, UMLClassifier *c)
   : UMLWidget(view, c) {
@@ -72,8 +70,8 @@ void ClassifierWidget::init() {
     m_bShowAttributes = ops.classState.showAtts;
     m_bShowStereotype = ops.classState.showStereoType;
     m_bDrawAsCircle = false;
-    setShowAttSigs( ops.classState.showAttSig );
     m_pAssocWidget = NULL;
+    setShowAttSigs( ops.classState.showAttSig );
 }
 
 void ClassifierWidget::updateSigs() {
@@ -302,7 +300,7 @@ QSize ClassifierWidget::calculateSize() {
     if (!m_pObject) {
         return UMLWidget::calculateSize();
     }
-    if (m_bDrawAsCircle) {
+    if (getClassifier()->isInterface() && m_bDrawAsCircle) {
         return calculateAsCircleSize();
     }
 
@@ -402,7 +400,7 @@ void ClassifierWidget::slotMenuSelection(int sel) {
         {
             UMLDoc *doc = UMLApp::app()->getDocument();
             Uml::Object_Type ot = ListPopupMenu::convert_MT_OT(mt);
-            if (doc->createChildObject(m_pObject, ot))
+            if (Object_Factory::createChildObject(getClassifier(), ot))
                 doc->setModified();
             updateComponentSize();
             update();
@@ -455,18 +453,12 @@ void ClassifierWidget::slotMenuSelection(int sel) {
 
     case ListPopupMenu::mt_ChangeToClass:
     case ListPopupMenu::mt_ChangeToClass_Selection:
-        WidgetBase::m_Type = Uml::wt_Class;
-        getClassifier()->setInterface(false);
-        updateComponentSize();
-        update();
+        changeToClass();
         break;
 
     case ListPopupMenu::mt_ChangeToInterface:
     case ListPopupMenu::mt_ChangeToInterface_Selection:
-        WidgetBase::m_Type = Uml::wt_Interface;
-        getClassifier()->setInterface(true);
-        updateComponentSize();
-        update();
+        changeToInterface();
         break;
 
     default:
@@ -532,7 +524,7 @@ void ClassifierWidget::draw(QPainter & p, int offsetX, int offsetY) {
     else
         p.setBrush( m_pView->viewport()->backgroundColor() );
 
-    if (m_bDrawAsCircle) {
+    if (getClassifier()->isInterface() && m_bDrawAsCircle) {
         drawAsCircle(p, offsetX, offsetY);
         return;
     }
@@ -729,6 +721,39 @@ void ClassifierWidget::toggleDrawAsCircle() {
     update();
 }
 
+void ClassifierWidget::changeToClass() {
+    WidgetBase::setBaseType(Uml::wt_Class);
+    getClassifier()->setInterface(false);
+
+    const Settings::OptionState& ops = m_pView->getOptionState();
+    m_bShowAttributes = ops.classState.showAtts;
+    m_bShowStereotype = ops.classState.showStereoType;
+
+    updateComponentSize();
+    update();
+}
+
+void ClassifierWidget::changeToInterface() {
+    WidgetBase::setBaseType(Uml::wt_Interface);
+    getClassifier()->setInterface(true);
+
+    m_bShowAttributes = false;
+    m_bShowStereotype = true;
+
+    updateComponentSize();
+    update();
+}
+
+void ClassifierWidget::adjustAssocs(int x, int y) {
+    UMLWidget::adjustAssocs(x, y);
+
+    if (m_pDoc->loading() || m_pAssocWidget == 0) {
+        return;
+    }
+
+    m_pAssocWidget->computeAssocClassLine();
+}
+
 void ClassifierWidget::saveToXMI(QDomDocument & qDoc, QDomElement & qElement) {
     QDomElement conceptElement;
     UMLClassifier *umlc = getClassifier();
