@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2002-2004
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2002-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                 *
  ***************************************************************************/
 #include <kdebug.h>
 #include <qdom.h>
@@ -20,11 +17,14 @@
 #include "../umldoc.h"
 #include "../umlview.h"
 #include "../umlobject.h"
+#include "../folder.h"
 #include "../classifier.h"
 #include "../umlwidget.h"
 #include "../umllistview.h"
 #include "../umllistviewitem.h"
 #include "../associationwidget.h"
+#include "../object_factory.h"
+#include "../model_utils.h"
 
 #define nfmt 4
 class UMLDragPrivate {
@@ -342,7 +342,7 @@ bool UMLDrag::decodeClip1(const QMimeSource* mimeSource, UMLObjectList& objects,
             element = objectElement.toElement();
             continue;
         }
-        pObject = UMLDoc::makeNewUMLObject(type);
+        pObject = Object_Factory::makeObjectFromXMI(type);
 
         if( !pObject ) {
             kdWarning() << "UMLDrag::decodeClip1: Given wrong type of umlobject to create: "
@@ -369,7 +369,8 @@ bool UMLDrag::decodeClip1(const QMimeSource* mimeSource, UMLObjectList& objects,
                 return false;
             }
             Uml::Object_Type type = pObject->getBaseType();
-            QString newName = doc->uniqObjectName(type, pObject->getName());
+            QString newName = Model_Utils::uniqObjectName(type, pObject->getUMLPackage(),
+                                                                pObject->getName());
             pObject->setName(newName);
             /****************************************************************/
 
@@ -473,7 +474,7 @@ bool UMLDrag::decodeClip2(const QMimeSource* mimeSource, UMLObjectList& objects,
         pObject = 0;
         QString type = element.tagName();
         if (type != "UML:Association") {
-            pObject = UMLDoc::makeNewUMLObject(type);
+            pObject = Object_Factory::makeObjectFromXMI(type);
 
             if( !pObject ) {
                 kdWarning() << "Given wrong type of umlobject to create:" << type << endl;
@@ -497,8 +498,20 @@ bool UMLDrag::decodeClip2(const QMimeSource* mimeSource, UMLObjectList& objects,
         kdWarning() << "no diagrams in XMI clip" << endl;
         return false;
     }
+    UMLListView *listView = UMLApp::app()->getListView();
     while ( !diagramElement.isNull() ) {
-        UMLView* view = new UMLView();
+        QString type = diagramElement.attribute("type", "0");
+        Uml::Diagram_Type dt = (Uml::Diagram_Type)type.toInt();
+        UMLListViewItem *parent = listView->findFolderForDiagram(dt);
+        if (parent == NULL)
+            return false;
+        UMLObject *po = parent->getUMLObject();
+        if (po == NULL || po->getBaseType() != Uml::ot_Folder) {
+            kdError() << "UMLDrag::decodeClip2: bad parent for view" << endl;
+            return false;
+        }
+        UMLFolder *f = static_cast<UMLFolder*>(po);
+        UMLView* view = new UMLView(f);
         view->loadFromXMI(diagramElement);
         diagrams.append(view);
         diagramNode = diagramNode.nextSibling();
@@ -513,7 +526,6 @@ bool UMLDrag::decodeClip2(const QMimeSource* mimeSource, UMLObjectList& objects,
         kdWarning() << "no listitems in XMI clip" << endl;
         return false;
     }
-    UMLListView *listView = UMLApp::app()->getListView();
     UMLListViewItem *currentItem = (UMLListViewItem*)listView->currentItem();
     while ( !listItemElement.isNull() ) {
         UMLListViewItem* itemData;
@@ -691,7 +703,7 @@ bool UMLDrag::decodeClip4(const QMimeSource* mimeSource, UMLObjectList& objects,
         //FIXME associations don't load
         if (type == "UML:Association")
             continue;
-        pObject = UMLDoc::makeNewUMLObject(type);
+        pObject = Object_Factory::makeObjectFromXMI(type);
 
         if ( !pObject ) {
             kdWarning() << "Given wrong type of umlobject to create: " << type << endl;
@@ -713,8 +725,9 @@ bool UMLDrag::decodeClip4(const QMimeSource* mimeSource, UMLObjectList& objects,
         kdWarning() << "no widgets in XMI clip" << endl;
         return false;
     }
+    UMLView *view = UMLApp::app()->getCurrentView();
     while ( !widgetElement.isNull() ) {
-        UMLWidget* widget = doc->getCurrentView()->loadWidgetFromXMI(widgetElement);
+        UMLWidget* widget = view->loadWidgetFromXMI(widgetElement);
         widgets.append(widget);
         widgetNode = widgetNode.nextSibling();
         widgetElement = widgetNode.toElement();
@@ -725,7 +738,6 @@ bool UMLDrag::decodeClip4(const QMimeSource* mimeSource, UMLObjectList& objects,
     QDomNode associationWidgetNode = associationWidgetsNode.firstChild();
     QDomElement associationWidgetElement = associationWidgetNode.toElement();
     while ( !associationWidgetElement.isNull() ) {
-        UMLView *view = UMLApp::app()->getCurrentView();
         AssociationWidget* associationWidget = new AssociationWidget(view);
         if (associationWidget->loadFromXMI(associationWidgetElement, widgets))
             associations.append(associationWidget);
@@ -794,12 +806,9 @@ bool UMLDrag::decodeClip5(const QMimeSource* mimeSource, UMLObjectList& objects,
     if ( element.isNull() ) {
         return false;//return ok as it means there is no umlobjects
     }
-    UMLObject* pObject = 0;
     while ( !element.isNull() ) {
-        pObject = 0;
         QString type = element.tagName();
-        pObject = UMLDoc::makeNewClassifierObject(type, newParent);
-
+        UMLClassifierListItem *pObject = newParent->makeChildObject(type);
         if( !pObject ) {
             kdWarning() << "Given wrong type of umlobject to create:" << type << endl;
             return false;
