@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2002-2004
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *  copyright (C) 2002-2006                                                *
+ *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
  ***************************************************************************/
 
 // own header
@@ -17,6 +14,7 @@
 
 // qt includes
 #include <qlayout.h>
+#include <qtooltip.h>
 #include <qvbox.h>
 // kde includes
 #include <kdebug.h>
@@ -28,8 +26,7 @@
 #include "codevieweroptionspage.h"
 #include "../dialog_utils.h"
 
-SettingsDlg::SettingsDlg( QWidget * parent, Settings::OptionState *state,
-                          QString activeLanguage, CodeGenerator * gen)
+SettingsDlg::SettingsDlg( QWidget * parent, Settings::OptionState *state )
         : KDialogBase( IconList, i18n("Umbrello Setup"),
                Help | Default | Apply | Ok | Cancel, Ok, parent, 0, true, true ) {
     m_bChangesApplied = false;
@@ -39,7 +36,7 @@ SettingsDlg::SettingsDlg( QWidget * parent, Settings::OptionState *state,
     setupFontPage();
     setupUIPage();
     setupClassPage();
-    setupCodeGenPage(gen, activeLanguage);
+    setupCodeGenPage();
     setupCodeViewerPage(state->codeViewerState);
 }
 
@@ -118,8 +115,15 @@ void SettingsDlg::setupGeneralPage() {
 
     m_GeneralWidgets.newcodegenCB = new QCheckBox( i18n("Use new C++/Java/Ruby generators"), m_GeneralWidgets.miscGB );
     m_GeneralWidgets.newcodegenCB -> setChecked( m_pOptionState->generalState.newcodegen );
+#ifndef BUG84739_FIXED
+    m_GeneralWidgets.newcodegenCB->setEnabled(false);
+#endif
     miscLayout -> addWidget( m_GeneralWidgets.newcodegenCB, 1, 0 );
 
+    m_GeneralWidgets.angularLinesCB = new QCheckBox( i18n("Use angular association lines"), m_GeneralWidgets.miscGB );
+    m_GeneralWidgets.angularLinesCB -> setChecked( m_pOptionState->generalState.angularlines );
+    miscLayout -> addWidget( m_GeneralWidgets.angularLinesCB, 1, 1 );
+
     //setup autosave settings
 
     m_GeneralWidgets.autosaveGB = new QGroupBox( i18n("Autosave"), page );
@@ -144,6 +148,14 @@ void SettingsDlg::setupGeneralPage() {
     Dialog_Utils::makeLabeledEditField( m_GeneralWidgets.autosaveGB, autosaveLayout, 2,
                                     m_GeneralWidgets.autosaveSuffixL, i18n("Set autosave suffix:"),
                                     m_GeneralWidgets.autosaveSuffixT, m_pOptionState->generalState.autosavesuffix );
+    QString autoSaveSuffixToolTip = i18n( "<qt><p>The autosave file will be saved to ~/autosave.xmi if the autosaving occurs "
+                                          "before you have manually saved the file.</p>"
+                                          "<p>If you've already saved it, the autosave file will be saved in the same folder as the file "
+                                          "and will be named like the file's name, followed by the suffix specified.</p>"
+                                          "<p>If the suffix is equal to the suffix of the file you've saved, "
+                                          "the autosave will overwrite your file automatically.</p></qt>" );
+    QToolTip::add( m_GeneralWidgets.autosaveSuffixL, autoSaveSuffixToolTip );
+    QToolTip::add( m_GeneralWidgets.autosaveSuffixT, autoSaveSuffixToolTip );
 
     //setup startup settings
     m_GeneralWidgets.startupGB = new QGroupBox( i18n("Startup"), page );
@@ -168,6 +180,7 @@ void SettingsDlg::setupGeneralPage() {
     startupLayout -> addWidget( m_GeneralWidgets.startL, 2, 0 );
 
     m_GeneralWidgets.diagramKB = new KComboBox( m_GeneralWidgets.startupGB );
+    m_GeneralWidgets.diagramKB->setCompletionMode( KGlobalSettings::CompletionPopup );
     startupLayout -> addWidget( m_GeneralWidgets.diagramKB, 2, 1 );
 
     QString diagrams [] = { i18n("No Diagram"), i18n("Class Diagram"),
@@ -178,13 +191,22 @@ void SettingsDlg::setupGeneralPage() {
 
     //start at 1 because we don't allow No Diagram any more
     for (int i=1; i<9; i++) {
-        m_GeneralWidgets.diagramKB->insertItem( diagrams[i] );
+        insertDiagram( diagrams[i] );
     }
 
     m_GeneralWidgets.diagramKB->setCurrentItem( (int)m_pOptionState->generalState.diagram-1 );
     connect( m_GeneralWidgets.autosaveCB, SIGNAL(clicked()), this, SLOT(slotAutosaveCBClicked()) );
 }
 
+/**
+* Inserts @p type into the type-combobox as well as its completion object.
+*/
+void SettingsDlg::insertDiagram( const QString& type, int index )
+{
+    m_GeneralWidgets.diagramKB->insertItem( type, index );
+    m_GeneralWidgets.diagramKB->completionObject()->addItem( type );
+}
+
 void SettingsDlg::setupClassPage() {
     //setup class settings page
 
@@ -236,26 +258,44 @@ void SettingsDlg::setupClassPage() {
     m_ClassWidgets.operationLabel = new QLabel( i18n("Default operation scope:"), m_ClassWidgets.scopeGB);
     scopeLayout -> addWidget( m_ClassWidgets.operationLabel, 1, 0 );
 
-    m_ClassWidgets.m_pAttribScopeCB = new QComboBox(m_ClassWidgets.scopeGB);
-    m_ClassWidgets.m_pAttribScopeCB->insertItem( tr2i18n( "Public" ) );
-    m_ClassWidgets.m_pAttribScopeCB->insertItem( tr2i18n( "Private" ) );
-    m_ClassWidgets.m_pAttribScopeCB->insertItem( tr2i18n( "Protected" ) );
+    m_ClassWidgets.m_pAttribScopeCB = new KComboBox(m_ClassWidgets.scopeGB);
+    insertAttribScope( tr2i18n( "Public" ) );
+    insertAttribScope( tr2i18n( "Private" ) );
+    insertAttribScope( tr2i18n( "Protected" ) );
     m_ClassWidgets.m_pAttribScopeCB->setCurrentItem((m_pOptionState->classState.defaultAttributeScope - 200));
+    m_ClassWidgets.m_pAttribScopeCB->setCompletionMode( KGlobalSettings::CompletionPopup );
     scopeLayout -> addWidget( m_ClassWidgets.m_pAttribScopeCB, 0, 1 );
 
-    m_ClassWidgets.m_pOperationScopeCB = new QComboBox(m_ClassWidgets.scopeGB);
-    m_ClassWidgets.m_pOperationScopeCB->insertItem( tr2i18n( "Public" ) );
-    m_ClassWidgets.m_pOperationScopeCB->insertItem( tr2i18n( "Private" ) );
-    m_ClassWidgets.m_pOperationScopeCB->insertItem( tr2i18n( "Protected" ) );
+    m_ClassWidgets.m_pOperationScopeCB = new KComboBox(m_ClassWidgets.scopeGB);
+    insertOperationScope( tr2i18n( "Public" ) );
+    insertOperationScope( tr2i18n( "Private" ) );
+    insertOperationScope( tr2i18n( "Protected" ) );
     m_ClassWidgets.m_pOperationScopeCB->setCurrentItem((m_pOptionState->classState.defaultOperationScope - 200));
+    m_ClassWidgets.m_pOperationScopeCB->setCompletionMode( KGlobalSettings::CompletionPopup );
     scopeLayout -> addWidget( m_ClassWidgets.m_pOperationScopeCB, 1, 1 );
 
 }
+/**
+* Inserts @p type into the type-combobox as well as its completion object.
+*/
+void SettingsDlg::insertAttribScope( const QString& type, int index )
+{
+    m_ClassWidgets.m_pAttribScopeCB->insertItem( type, index );
+    m_ClassWidgets.m_pAttribScopeCB->completionObject()->addItem( type );
+}
+/**
+* Inserts @p type into the type-combobox as well as its completion object.
+*/
+void SettingsDlg::insertOperationScope( const QString& type, int index )
+{
+    m_ClassWidgets.m_pOperationScopeCB->insertItem( type, index );
+    m_ClassWidgets.m_pOperationScopeCB->completionObject()->addItem( type );
+}
 
-void SettingsDlg::setupCodeGenPage(CodeGenerator *gen, const QString &activeLanguage) {
+void SettingsDlg::setupCodeGenPage() {
     //setup code generation settings page
     QVBox * page = addVBoxPage( i18n("Code Generation"), i18n("Code Generation Settings"), DesktopIcon( "source") );
-    m_pCodeGenPage = new CodeGenerationOptionsPage(gen, activeLanguage, page);
+    m_pCodeGenPage = new CodeGenerationOptionsPage(page);
     connect( m_pCodeGenPage, SIGNAL(languageChanged()), this, SLOT(slotApply()) );
 }
 
@@ -311,7 +351,7 @@ void SettingsDlg::slotDefault() {
     case Settings::page_UI:
         m_UiWidgets.useFillColorCB -> setChecked( true );
         m_UiWidgets.fillColorB -> setColor( QColor( 255, 255, 192 ) );
-        m_UiWidgets.lineColorB -> setColor( red );
+        m_UiWidgets.lineColorB -> setColor( Qt::red );
         m_UiWidgets.lineWidthB -> setValue( 0 );
         break;
 
@@ -341,6 +381,7 @@ void SettingsDlg::applyPage( Settings::Page page ) {
         m_pOptionState->generalState.undo = m_GeneralWidgets.undoCB -> isChecked();
         m_pOptionState->generalState.tabdiagrams = m_GeneralWidgets.tabdiagramsCB->isChecked();
         m_pOptionState->generalState.newcodegen = m_GeneralWidgets.newcodegenCB->isChecked();
+        m_pOptionState->generalState.angularlines = m_GeneralWidgets.angularLinesCB->isChecked();
         m_pOptionState->generalState.autosave = m_GeneralWidgets.autosaveCB -> isChecked();
         m_pOptionState->generalState.autosavetime = m_GeneralWidgets.timeISB -> value();
         // 2004-05-17 Achim Spangler: retrieve Suffix setting from dialog entry
@@ -348,7 +389,7 @@ void SettingsDlg::applyPage( Settings::Page page ) {
         m_pOptionState->generalState.logo = m_GeneralWidgets.logoCB -> isChecked();
         m_pOptionState->generalState.tip = m_GeneralWidgets.tipCB -> isChecked();
         m_pOptionState->generalState.loadlast = m_GeneralWidgets.loadlastCB -> isChecked();
-        m_pOptionState->generalState.diagram  = (Settings::Diagram)(m_GeneralWidgets.diagramKB->currentItem() + 1);
+        m_pOptionState->generalState.diagram  = (Uml::Diagram_Type)(m_GeneralWidgets.diagramKB->currentItem() + 1);
         break;
 
     case Settings::page_font:
@@ -386,7 +427,7 @@ void SettingsDlg::applyPage( Settings::Page page ) {
 }
 
 void SettingsDlg::slotLineBClicked() {
-    m_UiWidgets.lineColorB -> setColor( red );
+    m_UiWidgets.lineColorB -> setColor( Qt::red );
 }
 
 void SettingsDlg::slotFillBClicked() {
@@ -401,8 +442,4 @@ QString SettingsDlg::getCodeGenerationLanguage() {
     return m_pCodeGenPage->getCodeGenerationLanguage();
 }
 
-void SettingsDlg::setCodeGenerator(CodeGenerator *gen) {
-    m_pCodeGenPage->setCodeGenerator(gen);
-}
-
 #include "settingsdlg.moc"
