@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2002-2004
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2002-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                 *
  ***************************************************************************/
 
 // own header
@@ -48,6 +45,7 @@
 #include "../dialog_utils.h"
 #include "parmpropdlg.h"
 #include "../stereotype.h"
+#include "../uniqueid.h"
 
 UMLOperationDialog::UMLOperationDialog( QWidget * parent, UMLOperation * pOperation )
         : KDialogBase( Plain, i18n("Operation Properties"), Help | Ok | Cancel , Ok, parent, "_UMLOPERATIONDLG_", true, true) {
@@ -86,11 +84,14 @@ void UMLOperationDialog::setupDialog() {
     genLayout -> addWidget(m_pStereoTypeCB, 1, 1);
 
     m_pAbstractCB = new QCheckBox( i18n("&Abstract operation"), m_pGenGB );
-    m_pAbstractCB -> setChecked( m_pOperation -> getAbstract() );
-    m_pStaticCB = new QCheckBox( i18n("Classifier &scope (\"static\")"), m_pGenGB );
-    m_pStaticCB -> setChecked( m_pOperation -> getStatic() );
+    m_pAbstractCB -> setChecked( m_pOperation->getAbstract() );
     genLayout -> addWidget( m_pAbstractCB, 2, 0 );
+    m_pStaticCB = new QCheckBox( i18n("Classifier &scope (\"static\")"), m_pGenGB );
+    m_pStaticCB -> setChecked( m_pOperation->getStatic() );
     genLayout -> addWidget( m_pStaticCB, 2, 1 );
+    m_pQueryCB = new QCheckBox( i18n("&Query (\"const\")"), m_pGenGB );
+    m_pQueryCB -> setChecked( m_pOperation->getConst() );
+    genLayout -> addWidget( m_pQueryCB, 2, 2 );
 
     topLayout -> addWidget( m_pGenGB );
 
@@ -155,6 +156,7 @@ void UMLOperationDialog::setupDialog() {
     insertType( "void" );
 
     m_pRtypeCB->setDuplicatesEnabled(false);//only allow one of each type in box
+    m_pRtypeCB->setCompletionMode( KGlobalSettings::CompletionPopup );
 
     // add template parameters
     UMLClassifier *classifier = dynamic_cast<UMLClassifier*>(m_pOperation->parent());
@@ -207,6 +209,7 @@ void UMLOperationDialog::setupDialog() {
 
     // manage stereotypes
     m_pStereoTypeCB -> setDuplicatesEnabled(false);//only allow one of each type in box
+    m_pStereoTypeCB->setCompletionMode( KGlobalSettings::CompletionPopup );
     insertStereotype (QString("")); // an empty stereotype is the default
     int defaultStereotype=0;
     bool foundDefaultStereotype = false;
@@ -324,7 +327,7 @@ void UMLOperationDialog::slotNewParameter() {
             m_pOperation->addParm( dlg.getTypeName(), name, dlg.getInitialValue(),
                                dlg.getDoc(), dlg.getParmKind() );
              */
-            newAttribute->setID( m_doc->getUniqueID() );
+            newAttribute->setID( UniqueID::gen() );
             newAttribute->setName( name );
             newAttribute->setTypeName( dlg.getTypeName() );
             newAttribute->setInitialValue( dlg.getInitialValue() );
@@ -485,6 +488,8 @@ bool UMLOperationDialog::apply()
     else
         m_pOperation->setTypeName(typeName);
 
+    m_pOperation->setStereotype( m_pStereoTypeCB->currentText() );
+
     bool isAbstract = m_pAbstractCB->isChecked();
     m_pOperation -> setAbstract( isAbstract );
     if (isAbstract) {
@@ -495,8 +500,8 @@ bool UMLOperationDialog::apply()
          */
         classifier->setAbstract(true);
     }
-    m_pOperation -> setStatic( m_pStaticCB -> isChecked() );
-    m_pOperation -> setStereotype( m_pStereoTypeCB->currentText() );
+    m_pOperation->setStatic( m_pStaticCB->isChecked() );
+    m_pOperation->setConst( m_pQueryCB->isChecked() );
 
     return true;
 }
