@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2003-2005
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *  copyright (C) 2003-2006                                                *
+ *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
  ***************************************************************************/
 
 // own header
@@ -22,6 +19,7 @@
 #include "entityattribute.h"
 #include "umldoc.h"
 #include "uml.h"
+#include "uniqueid.h"
 #include "clipboard/idchangelog.h"
 #include "dialogs/umlentityattributedialog.h"
 
@@ -54,27 +52,26 @@ void UMLEntity::init() {
     m_BaseType = Uml::ot_Entity;
 }
 
-UMLObject* UMLEntity::createEntityAttribute(const QString &name /*=null*/) {
-    UMLDoc *umldoc = UMLApp::app()->getDocument();
-    Uml::IDType id = umldoc->getUniqueID();
+UMLAttribute* UMLEntity::createAttribute(const QString &name /*=null*/) {
+    Uml::IDType id = UniqueID::gen();
     QString currentName;
     if (name.isNull())  {
         currentName = uniqChildName(Uml::ot_EntityAttribute);
     } else {
         currentName = name;
     }
-    const Settings::OptionState optionState = UMLApp::app()->getOptionState();
+    const Settings::OptionState optionState = Settings::getOptionState();
     Uml::Visibility scope = optionState.classState.defaultAttributeScope;
     UMLEntityAttribute* newAttribute = new UMLEntityAttribute(this, currentName, id, scope);
 
     int button = QDialog::Accepted;
     bool goodName = false;
 
-    //check for name.isNull() stops dialogue being shown
+    //check for name.isNull() stops dialog being shown
     //when creating attribute via list view
     while (button==QDialog::Accepted && !goodName && name.isNull()) {
-        UMLEntityAttributeDialog attributeDialogue(0, newAttribute);
-        button = attributeDialogue.exec();
+        UMLEntityAttributeDialog attributedialog(0, newAttribute);
+        button = attributedialog.exec();
         QString name = newAttribute->getName();
 
         if(name.length() == 0) {
@@ -92,6 +89,7 @@ UMLObject* UMLEntity::createEntityAttribute(const QString &name /*=null*/) {
 
     addEntityAttribute(newAttribute);
 
+    UMLDoc *umldoc = UMLApp::app()->getDocument();
     umldoc->signalUMLObjectCreated(newAttribute);
     return newAttribute;
 }
@@ -140,36 +138,41 @@ bool UMLEntity::addEntityAttribute(UMLEntityAttribute* attribute, int position)
     return false;
 }
 
-int UMLEntity::removeEntityAttribute(UMLObject* literal) {
+int UMLEntity::removeEntityAttribute(UMLClassifierListItem* literal) {
     if (!m_List.remove((UMLEntityAttribute*)literal)) {
         kdDebug() << "can't find att given in list" << endl;
         return -1;
     }
     emit entityAttributeRemoved(literal);
     emit modified();
-    // If we are deleteing the object, then we dont need to disconnect..this is done auto-magically
+    // If we are deleteing the object, then we don't need to disconnect..this is done auto-magically
     // for us by QObject. -b.t.
     // disconnect(a,SIGNAL(modified()),this,SIGNAL(modified()));
     delete literal;
     return m_List.count();
 }
 
-UMLEntityAttribute* UMLEntity::takeEntityAttribute(UMLEntityAttribute* el, int *wasAtIndex) {
-    int index = m_List.findRef( el );
-    if (wasAtIndex)
-        *wasAtIndex = index;
-    el = (index == -1 ? 0 : dynamic_cast<UMLEntityAttribute*>(m_List.take( )));
-    if (el) {
-        emit entityAttributeRemoved(el);
-        emit modified();
-    }
-    return el;
-}
-
 int UMLEntity::entityAttributes() {
-    return m_List.count();
+    UMLClassifierListItemList entityAttributes = getFilteredList(Uml::ot_EntityAttribute);
+    return entityAttributes.count();
 }
 
+void UMLEntity::signalEntityAttributeRemoved(UMLClassifierListItem *eattr) {
+    emit entityAttributeRemoved(eattr);
+}
+
+bool UMLEntity::resolveRef() {
+    bool success = UMLClassifier::resolveRef();
+    for (UMLObjectListIt oit(m_List); oit.current(); ++oit) {
+        UMLObject* obj = oit.current();
+        if (obj->resolveRef()) {
+            UMLClassifierListItem *cli = static_cast<UMLClassifierListItem*>(obj);
+            if (cli->getBaseType() == Uml::ot_EntityAttribute)
+                emit entityAttributeAdded(cli);
+        }
+    }
+    return success;
+}
 
 void UMLEntity::saveToXMI(QDomDocument& qDoc, QDomElement& qElement) {
     QDomElement entityElement = UMLObject::save("UML:Entity", qDoc);
