@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2003-2005
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *  copyright (C) 2003-2006                                                *
+ *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef ENTITY_H
@@ -25,11 +22,6 @@ class UMLEntityAttribute;
  * This class inherits from @ref UMLClassifier which contains most of the
  * information.
  *
- * The @ref UMLDoc class creates instances of this type.  All Entitys
- * will need a unique id.  This will be given by the @ref UMLDoc class.
- * If you don't leave it up to the @ref UMLDoc class then call the method
- * @ref UMLDoc::getUniqueID to get a unique id.
- *
  * @short Non-graphical Information for an Entity.
  * @author Jonathan Riddell
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
@@ -68,12 +60,13 @@ public:
 
     /**
      * Creates an entity attribute for the parent concept.
+     * Reimplementation of method from UMLClassifier.
      *
      * @param o The parent concept
      * @param name  An optional name, used by when creating through UMLListView
      * @return  The UMLEntityAttribute created
      */
-    UMLObject* createEntityAttribute(const QString &name = QString::null);
+    UMLAttribute* createAttribute(const QString &name = QString::null);
 
     /**
          * Adds an entityAttribute to the entity.
@@ -91,7 +84,7 @@ public:
          *
          * @param Att           Pointer to the UMLEntityAttribute.
      * @param Log               Pointer to the IDChangeLog.
-     * @return  True if the entityAttribute was sucessfully added.
+     * @return  True if the entityAttribute was successfully added.
      */
     bool addEntityAttribute(UMLEntityAttribute* att, IDChangeLog* Log = 0);
 
@@ -102,7 +95,7 @@ public:
          *
          * @param Att           Pointer to the UMLEntityAttribute.
      * @param position  Position index for the insertion.
-     * @return  True if the entityAttribute was sucessfully added.
+     * @return  True if the entityAttribute was successfully added.
      */
     //TODO:  give default value -1 to position (append) - now it conflicts with the method above..
     bool addEntityAttribute(UMLEntityAttribute* att, int position );
@@ -114,19 +107,12 @@ public:
      * @return  Count of the remaining entityAttributes after removal.
      *          Returns -1 if the given entityAttribute was not found.
      */
-    int removeEntityAttribute(UMLObject* a);
+    int removeEntityAttribute(UMLClassifierListItem* a);
 
     /**
-     * Take and return an entityAttribute from class.
-     * It is the callers responsibility to pass on ownership of
-     * the returned entityAttribute (or to delete the entityAttribute)
-     *
-     * @param  el entityAttribute to tkae
-     * @param wasAtIndex        if given, the index in m_List of the item taken
-     *                  is returned in the int pointed-to.
-     * @return pointer to the entityAttribute or null if not found.
+     * Emit the entityAttributeRemoved signal.
      */
-    UMLEntityAttribute* takeEntityAttribute(UMLEntityAttribute* el, int *wasAtIndex = NULL);
+    void signalEntityAttributeRemoved(UMLClassifierListItem *eattr);
 
     /**
      * Returns the number of entityAttributes for the class.
@@ -135,14 +121,20 @@ public:
      */
     int entityAttributes();
 
+    /**
+     * Resolve the types referenced by our UMLEntityAttributes.
+     * Reimplements the method from UMLClassifier.
+     */
+    virtual bool resolveRef();
+
     /**
      * Creates the <UML:Entity> element including its entityliterals.
      */
     virtual void saveToXMI(QDomDocument& qDoc, QDomElement& qElement);
 
 signals:
-    void entityAttributeAdded(UMLObject*);
-    void entityAttributeRemoved(UMLObject*);
+    void entityAttributeAdded(UMLClassifierListItem*);
+    void entityAttributeRemoved(UMLClassifierListItem*);
 
 protected:
     /**
