@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2002-2005
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2002-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                 *
  ***************************************************************************/
 
 //qt includes
@@ -19,19 +16,21 @@
 #include <kcursor.h>
 //app includes
 #include "messagewidget.h"
-#include "floatingtext.h"
+#include "messagewidgetcontroller.h"
+#include "floatingtextwidget.h"
 #include "objectwidget.h"
 #include "classifier.h"
 #include "operation.h"
 #include "umlview.h"
 #include "umldoc.h"
 #include "uml.h"
+#include "uniqueid.h"
 #include "listpopupmenu.h"
 
 MessageWidget::MessageWidget(UMLView * view, ObjectWidget* a, ObjectWidget* b,
                              int y, Uml::Sequence_Message_Type sequenceMessageType,
                              Uml::IDType id /* = Uml::id_None */)
-        : UMLWidget(view, id) {
+        : UMLWidget(view, id, new MessageWidgetController(this)) {
     init();
     m_pOw[Uml::A] = a;
     m_pOw[Uml::B] = b;
@@ -41,17 +40,17 @@ MessageWidget::MessageWidget(UMLView * view, ObjectWidget* a, ObjectWidget* b,
         y -= m_pOw[Uml::B]->getHeight() / 2;
         m_pOw[Uml::B]->setY(y);
     }
-
+    updateResizability();
     calculateWidget();
-    y = y < getMinHeight() ? getMinHeight() : y;
-    y = y > getMaxHeight() ? getMaxHeight() : y;
+    y = y < getMinY() ? getMinY() : y;
+    y = y > getMaxY() ? getMaxY() : y;
     m_nY = y;
 
     this->activate();
 }
 
 MessageWidget::MessageWidget(UMLView * view, Uml::Sequence_Message_Type seqMsgType, Uml::IDType id)
-        : UMLWidget(view, id) {
+        : UMLWidget(view, id, new MessageWidgetController(this)) {
     init();
     m_sequenceMessageType = seqMsgType;
 }
@@ -63,13 +62,20 @@ void MessageWidget::init() {
     m_pOw[Uml::A] = m_pOw[Uml::B] = NULL;
     m_pFText = NULL;
     m_nY = 0;
-    m_inSelection = false;
     setVisible(true);
 }
 
 MessageWidget::~MessageWidget() {
 }
 
+void MessageWidget::updateResizability() {
+    if (m_sequenceMessageType == Uml::sequence_message_synchronous ||
+        m_pOw[Uml::A] == m_pOw[Uml::B])
+        UMLWidget::m_bResizable = true;
+    else
+        UMLWidget::m_bResizable = false;
+}
+
 void MessageWidget::draw(QPainter& p, int offsetX, int offsetY) {
     if(!m_pOw[Uml::A] || !m_pOw[Uml::B]) {
         return;
@@ -213,7 +219,7 @@ void MessageWidget::drawAsynchronous(QPainter& p, int offsetX, int offsetY) {
     }
 
     if (m_bSelected)
-        drawSelected(&p, offsetX, offsetY, m_pOw[Uml::A] == m_pOw[Uml::B]);
+        drawSelected(&p, offsetX, offsetY);
 }
 
 void MessageWidget::drawCreation(QPainter& p, int offsetX, int offsetY) {
@@ -242,11 +248,11 @@ void MessageWidget::drawCreation(QPainter& p, int offsetX, int offsetY) {
     }
 
     if (m_bSelected)
-        drawSelected(&p, offsetX, offsetY, false);
+        drawSelected(&p, offsetX, offsetY);
 }
 
 int MessageWidget::onWidget(const QPoint & p) {
-    if (m_sequenceMessageType == Uml::sequence_message_asynchronous) {
+    if (m_sequenceMessageType != Uml::sequence_message_synchronous) {
         return UMLWidget::onWidget(p);
     }
     // Synchronous message:
@@ -272,8 +278,9 @@ void MessageWidget::setTextPosition() {
         << endl;
         return;
     }
-    if (m_pFText->getText().isEmpty())
+    if (m_pFText->getDisplayText().isEmpty()) {
         return;
+    }
     m_pFText->updateComponentSize();
     int ftX = constrainX(m_pFText->getX(), m_pFText->getWidth(), m_pFText->getRole());
     int ftY = getY() - m_pFText->getHeight();
@@ -306,13 +313,13 @@ void MessageWidget::constrainTextPos(int &textX, int &textY, int textWidth, int
                                      Uml::Text_Role tr) {
     textX = constrainX(textX, textWidth, tr);
     // Constrain Y.
-    const int minTextY = getMinHeight();
-    const int maxTextY = getMaxHeight() - textHeight - 5;
+    const int minTextY = getMinY();
+    const int maxTextY = getMaxY() - textHeight - 5;
     if (textY < minTextY)
         textY = minTextY;
     else if (textY > maxTextY)
         textY = maxTextY;
-    setY( textY + textHeight );   // NB: side effect
+//     setY( textY + textHeight );   // NB: side effect
 }
 
 void MessageWidget::setLinkAndTextPos() {
@@ -327,15 +334,17 @@ void MessageWidget::moveEvent(QMoveEvent* /*m*/) {
     if (!m_pFText) {
         return;
     }
-    if (m_pView->getSelectCount() > 2) {
+    //TODO why this condition?
+/*    if (m_pView->getSelectCount() > 2) {
         return;
-    }
+    }*/
+
     setTextPosition();
 
     emit sigMessageMoved();
 }
 
-void MessageWidget::resizeEvent(QResizeEvent */*re*/) {
+void MessageWidget::resizeEvent(QResizeEvent* /*re*/) {
 }
 
 void MessageWidget::calculateWidget() {
@@ -349,15 +358,23 @@ void MessageWidget::calculateWidget() {
 }
 
 void MessageWidget::slotWidgetMoved(Uml::IDType id) {
-    if(m_pOw[Uml::A] -> getLocalID() == id || m_pOw[Uml::B] -> getLocalID() == id) {
-        m_nY = getY();
-        m_nY = m_nY < getMinHeight()?getMinHeight():m_nY;
-        m_nY = m_nY > getMaxHeight()?getMaxHeight():m_nY;
-        calculateWidget();
+    const Uml::IDType idA = m_pOw[Uml::A]->getLocalID();
+    const Uml::IDType idB = m_pOw[Uml::B]->getLocalID();
+    if (idA != id && idB != id) {
+        kdDebug() << "MessageWidget::slotWidgetMoved(" << ID2STR(id)
+            << "): ignoring for idA=" << ID2STR(idA)
+            << ", idB=" << ID2STR(idB) << endl;
+        return;
     }
+    m_nY = getY();
+    if (m_nY < getMinY())
+        m_nY = getMinY();
+    if (m_nY > getMaxY())
+        m_nY = getMaxY();
+    calculateWidget();
     if( !m_pFText )
         return;
-    if( m_pView -> getSelectCount() > 1 )
+    if (m_pView->getSelectCount(true) > 1)
         return;
     setTextPosition();
 }
@@ -378,7 +395,7 @@ void MessageWidget::slotMenuSelection(int sel) {
             Uml::Text_Role tr = Uml::tr_Seq_Message;
             if (m_pOw[Uml::A] == m_pOw[Uml::B])
                 tr = Uml::tr_Seq_Message_Self;
-            m_pFText = new FloatingText( m_pView, tr );
+            m_pFText = new FloatingTextWidget( m_pView, tr );
             m_pFText->setFont(UMLWidget::getFont());
             setLinkAndTextPos();
             m_pView->getWidgetList().append(m_pFText);
@@ -387,12 +404,6 @@ void MessageWidget::slotMenuSelection(int sel) {
     }
 }
 
-void MessageWidget::mouseDoubleClickEvent(QMouseEvent * /*me*/) {
-    if (m_pView->getCurrentCursor() == WorkToolBar::tbb_Arrow &&
-            m_pFText != NULL)
-        m_pFText -> slotMenuSelection(ListPopupMenu::mt_Select_Operation);
-}
-
 void MessageWidget::activate(IDChangeLog * Log /*= 0*/) {
     m_pView->resetPastePoint();
     UMLWidget::activate(Log);
@@ -404,7 +415,7 @@ void MessageWidget::activate(IDChangeLog * Log /*= 0*/) {
         Uml::Text_Role tr = Uml::tr_Seq_Message;
         if (m_pOw[Uml::A] == m_pOw[Uml::B])
             tr = Uml::tr_Seq_Message_Self;
-        m_pFText = new FloatingText( m_pView, tr, "" );
+        m_pFText = new FloatingTextWidget( m_pView, tr, "" );
         m_pFText->setFont(UMLWidget::getFont());
     }
     setLinkAndTextPos();
@@ -425,7 +436,7 @@ void MessageWidget::activate(IDChangeLog * Log /*= 0*/) {
     emit sigMessageMoved();
 }
 
-void MessageWidget::setMessageText(FloatingText *ft) {
+void MessageWidget::setMessageText(FloatingTextWidget *ft) {
     if (ft == NULL)
         return;
     QString displayText = m_SequenceNumber + ": " + getOperationText(m_pView);
@@ -433,7 +444,7 @@ void MessageWidget::setMessageText(FloatingText *ft) {
     setTextPosition();
 }
 
-void MessageWidget::setText(FloatingText *ft, const QString &newText) {
+void MessageWidget::setText(FloatingTextWidget *ft, const QString &newText) {
     ft->setText(newText);
     UMLApp::app()->getDocument()->setModified(true);
 }
@@ -468,7 +479,11 @@ UMLOperation *MessageWidget::getOperation() {
 }
 
 void MessageWidget::setOperation(UMLOperation *op) {
+    if (m_pObject && m_pFText)
+        disconnect(m_pObject, SIGNAL(modified()), m_pFText, SLOT(setMessageText()));
     m_pObject = op;
+    if (m_pObject && m_pFText)
+        connect(m_pObject, SIGNAL(modified()), m_pFText, SLOT(setMessageText()));
 }
 
 QString MessageWidget::getCustomOpText() {
@@ -514,7 +529,7 @@ void MessageWidget::calculateDimensionsSynchronous() {
     int x1 = m_pOw[Uml::A]->getX();
     int x2 = m_pOw[Uml::B]->getX();
     int w1 = m_pOw[Uml::A]->getWidth() / 2;
-    int w2 =  m_pOw[Uml::B]->getWidth() / 2;
+    int w2 = m_pOw[Uml::B]->getWidth() / 2;
     x1 += w1;
     x2 += w2;
 
@@ -546,8 +561,8 @@ void MessageWidget::calculateDimensionsAsynchronous() {
 
     int x1 = m_pOw[Uml::A]->getX();
     int x2 = m_pOw[Uml::B]->getX();
-    int w1 = m_pOw[Uml::A] -> getWidth() / 2;
-    int w2 =  m_pOw[Uml::B] -> getWidth() / 2;
+    int w1 = m_pOw[Uml::A]->getWidth() / 2;
+    int w2 = m_pOw[Uml::B]->getWidth() / 2;
     x1 += w1;
     x2 += w2;
 
@@ -579,8 +594,8 @@ void MessageWidget::calculateDimensionsCreation() {
 
     int x1 = m_pOw[Uml::A]->getX();
     int x2 = m_pOw[Uml::B]->getX();
-    int w1 = m_pOw[Uml::A] -> getWidth() / 2;
-    int w2 =  m_pOw[Uml::B] -> getWidth();
+    int w1 = m_pOw[Uml::A]->getWidth() / 2;
+    int w2 = m_pOw[Uml::B]->getWidth();
     x1 += w1;
     if (x1 > x2)
         x2 += w2;
@@ -597,6 +612,7 @@ void MessageWidget::calculateDimensionsCreation() {
     x += 1;
     widgetWidth -= 2;
     m_nPosX = x;
+    m_nY = m_pOw[Uml::B]->getY() + m_pOw[Uml::B]->getHeight() / 2;
     setSize(widgetWidth, widgetHeight);
 }
 
@@ -617,92 +633,26 @@ void MessageWidget::cleanup() {
     }
 }
 
-void MessageWidget::mouseMoveEvent(QMouseEvent *me) {
-    int newX = 0, newY = 0;
-    int moveX, moveY;
-    if( m_bResizing ) {
-        int heightA = (int)((ObjectWidget*)m_pOw[Uml::A]) -> getEndLineY();
-        int heightB = (int)((ObjectWidget*)m_pOw[Uml::B]) -> getEndLineY();
-        int height = heightA;
-        if ( heightA > heightB )  {
-            height = heightB;
-        }
-        moveY = (int)me -> y() - m_nPressOffsetY - m_nOldY;
-        newY = m_nOldH + moveY;
-        newY = newY < 20 ? 20 : newY;
-        setSize( width(), newY );
-        emit sigMessageMoved();
-        return;
-    }
-    if( !m_bSelected )
-        m_pView -> setSelected( this, me );
-    m_bSelected = true;
-    if( !m_bMouseDown )
-        if( me -> button() != Qt::LeftButton )
-            return;
-    int count = m_pView -> getSelectCount();
-
-    //If not m_bStartMove means moving as part of selection
-    //me->pos() will have the amount we need to move.
-    if(!m_bStartMove) {
-        moveX = (int)me -> x();
-        moveY = (int)me -> y();
-    } else {
-        //we started the move so..
-        //move any others we are selected
-        moveX = (int)me -> x() - m_nOldX - m_nPressOffsetX;
-        moveY = (int)me -> y() - m_nOldY - m_nPressOffsetY;
-        if( (getX() + moveX) < 0 )
-            moveX = 0;
-        if( ( getY() + moveY) < 0 )
-            moveY = 0;
-        if( count > 2 )
-            m_pView -> moveSelected( this, moveX, 0 );
-    }
-    newY = getY() + moveY;
-    newY = newY < 0?0:newY;
-    if( count > 2 )
-        newY = getY();  //only change y if not selected
-    newY = newY < getMinHeight() ? getMinHeight() : newY;
-    newY = newY > getMaxHeight() ? getMaxHeight() : newY;
-
-    if (m_nOldX != newX || m_nOldY != newY) {
-        m_bMoved = true;
-    }
-    m_nOldY = newY;
-    setY( newY );
-    adjustAssocs(newX, newY);
-    if (m_sequenceMessageType == Uml::sequence_message_creation) {
-        const int objWidgetHalfHeight = m_pOw[Uml::B]->getHeight() / 2;
-        m_pOw[Uml::B]->UMLWidget::setY( newY - objWidgetHalfHeight );
-    }
-    moveEvent(0);
-}
-
 void MessageWidget::setSelected(bool _select) {
-    if( m_inSelection )//used to stop a recursive call
-    {
-        m_inSelection = false;
-        return;
-    }
     UMLWidget::setSelected( _select );
-    if( !m_pFText )
+    if( !m_pFText || m_pFText->getDisplayText().isEmpty())
         return;
     if( m_bSelected && m_pFText -> getSelected() )
         return;
     if( !m_bSelected && !m_pFText -> getSelected() )
         return;
 
-    m_inSelection = true;
     m_pView -> setSelected( m_pFText, 0 );
     m_pFText -> setSelected( m_bSelected );
 }
 
-int MessageWidget::getMinHeight() {
-    if (!m_pOw[Uml::A] || !m_pOw[Uml::B] ||
-            m_sequenceMessageType == Uml::sequence_message_creation) {
+int MessageWidget::getMinY() {
+    if (!m_pOw[Uml::A] || !m_pOw[Uml::B]) {
         return 0;
     }
+    if (m_sequenceMessageType == Uml::sequence_message_creation) {
+        return m_pOw[Uml::A]->getY() + m_pOw[Uml::A]->getHeight();
+    }
     int heightA = m_pOw[Uml::A]->getY() + m_pOw[Uml::A]->getHeight();
     int heightB = m_pOw[Uml::B]->getY() + m_pOw[Uml::B]->getHeight();
     int height = heightA;
@@ -712,7 +662,7 @@ int MessageWidget::getMinHeight() {
     return height;
 }
 
-int MessageWidget::getMaxHeight() {
+int MessageWidget::getMaxY() {
     if( !m_pOw[Uml::A] || !m_pOw[Uml::B] ) {
         return 0;
     }
@@ -725,17 +675,9 @@ int MessageWidget::getMaxHeight() {
     return (height - this->height());
 }
 
-void MessageWidget::mousePressEvent(QMouseEvent* me) {
-    // resize only applies to asynchronous messages between different objects
-    if ( m_sequenceMessageType == Uml::sequence_message_asynchronous &&
-            m_pOw[Uml::A] != m_pOw[Uml::B] ) {
-        return;
-    }
-    UMLWidget::mousePressEvent(me);
-}
-
 void MessageWidget::setWidget(ObjectWidget * ow, Uml::Role_Type role) {
     m_pOw[role] = ow;
+    updateResizability();
 }
 
 ObjectWidget* MessageWidget::getWidget(Uml::Role_Type role) {
@@ -803,33 +745,18 @@ bool MessageWidget::loadFromXMI(QDomElement& qElement) {
         << ID2STR(bId) << " is not an ObjectWidget" << endl;
         return false;
     }
+    updateResizability();
 
     UMLClassifier *c = dynamic_cast<UMLClassifier*>( pWB->getUMLObject() );
+    UMLOperation *op = NULL;
     if (c) {
         Uml::IDType opId = STR2ID(m_CustomOp);
-        UMLOperation *op = dynamic_cast<UMLOperation*>( c->findChildObjectById(opId, true) );
+        op = dynamic_cast<UMLOperation*>( c->findChildObjectById(opId, true) );
         if (op) {
             // If the UMLOperation is set, m_CustomOp isn't used anyway.
             // Just setting it empty for the sake of sanity.
             m_CustomOp = QString::null;
-        } else {
-            // Previous umbrello versions saved the operation text
-            // instead of the xmi.id of the operation.
-            // For backward compatibility, attempt to determine the
-            // m_pOperation from the operation text:
-            Model_Utils::OpDescriptor od;
-            Model_Utils::Parse_Status st = Model_Utils::parseOperation(m_CustomOp, od, c);
-            if (st == Model_Utils::PS_OK) {
-                bool isExistingOp = false;
-                UMLObject *o = c->createOperation(od.m_name, &isExistingOp, &od.m_args);
-                op = static_cast<UMLOperation*>(o);
-                if (od.m_pReturnType) {
-                    op->setType(od.m_pReturnType);
-                }
-                m_CustomOp = QString::null;
-            }
         }
-        setOperation(op);
     }
 
     Uml::IDType textId = STR2ID(textid);
@@ -838,13 +765,13 @@ bool MessageWidget::loadFromXMI(QDomElement& qElement) {
         if (flotext != NULL) {
             // This only happens when loading files produced by
             // umbrello-1.3-beta2.
-            m_pFText = static_cast<FloatingText*>(flotext);
+            m_pFText = static_cast<FloatingTextWidget*>(flotext);
             setLinkAndTextPos();
             return true;
         }
     } else {
         // no textid stored -> get unique new one
-        textId = UMLApp::app()->getDocument()->getUniqueID();
+        textId = UniqueID::gen();
     }
 
     Uml::Text_Role tr = Uml::tr_Seq_Message;
@@ -857,9 +784,9 @@ bool MessageWidget::loadFromXMI(QDomElement& qElement) {
     if ( !element.isNull() ) {
         QString tag = element.tagName();
         if (tag == "floatingtext") {
-            m_pFText = new FloatingText( m_pView, tr, getOperationText(m_pView), textId );
+            m_pFText = new FloatingTextWidget( m_pView, tr, getOperationText(m_pView), textId );
             if( ! m_pFText->loadFromXMI(element) ) {
-                // Most likely cause: The FloatingText is empty.
+                // Most likely cause: The FloatingTextWidget is empty.
                 delete m_pFText;
                 m_pFText = NULL;
             }
@@ -868,6 +795,8 @@ bool MessageWidget::loadFromXMI(QDomElement& qElement) {
             << tag << endl;
         }
     }
+    if (op)                // Do it here and not earlier because now we have the
+        setOperation(op);  // m_pFText and setOperation() can make connections.
 
     // always need this
     setLinkAndTextPos();
