@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2002-2004
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2002-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                 *
  ***************************************************************************/
 
 #ifndef UML_H
@@ -19,8 +16,7 @@
 #include <config.h>
 #endif
 
-#include "optionstate.h"
-#include "generatorinfo.h"
+#include "umlnamespace.h"
 
 #include <qmap.h>
 #include <qdict.h>
@@ -34,17 +30,18 @@ class AlignToolBar;
 class CodeDocument;
 class CodeGenerator;
 class CodeGenerationPolicy;
+class CodeGenPolicyExt;
 class DocWindow;
 class UMLClassifier;
 class UMLDoc;
 class UMLListView;
 class UMLView;
 class WorkToolBar;
-class InfoWidget;
 class SettingsDlg;
-class ExportViewAction;
+class UMLViewImageExporterAll;
 class RefactoringAssistant;
 class KPlayerPopupSliderAction;
+class XhtmlGenerator;
 
 // KDE forward declarations
 class KActionMenu;
@@ -61,11 +58,12 @@ class QWidgetStack;
 class QMenuData;
 class QClipboard;
 class QToolButton;
+class QCustomEvent;
 
 /**
  * The base class for UML application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
- * and statusbar. An instance of UMLView creates your centre view, which is connected
+ * and statusbar. An instance of UMLView creates your center view, which is connected
  * to the window's Doc object.
  * UMLApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
@@ -197,7 +195,7 @@ public:
      *
      * @return  Pointer to the CodeGenerator.
      */
-    CodeGenerator* getGenerator(bool warnMissing = true);
+    CodeGenerator* getGenerator();
 
     /**
      * Set the current generator for this app.
@@ -207,22 +205,24 @@ public:
      *
      * @param gen               Pointer to the CodeGenerator to set.
      * @param giveWarning       True to enable out-of-date warning.
-            */
+     */
     void setGenerator(CodeGenerator* gen, bool giveWarning = true);
 
     /**
      * Creates a new code generator for the given active language.
      *
      * @return  Pointer to the CodeGenerator created.
-         */
+     */
     CodeGenerator* createGenerator();
 
     /**
-     * Returns the dictionary of available languages.
-     *
-     * @return  Pointer to the dictionary of available languages.
+     * Auxiliary function for UMLDoc::loadExtensionsFromXMI():
+     * Return the code generator of the given language if it already
+     * exists; if it does not yet exist then create it and return
+     * the newly created generator. It is the caller's responsibility
+     * to load XMI into the newly created generator.
      */
-    GeneratorDict& generatorDict();
+    CodeGenerator *setGenerator(Uml::Programming_Language pl);
 
     /**
      * Call the refactoring assistant on a classifier.
@@ -253,12 +253,11 @@ public:
 
     /**
      * Puts this view to the top of the viewStack, i.e. makes it
-     * visible to the user.  If no view is specified the blank
-     * infoWidget is shown instead.
+     * visible to the user.
      *
      * @param view              Pointer to the UMLView to push.
      */
-    void setCurrentView(UMLView* view = 0);
+    void setCurrentView(UMLView* view);
 
     /**
      * Get the current view.
@@ -269,20 +268,20 @@ public:
     UMLView* getCurrentView();
 
     /**
-     * Sets the default mimetype for all diagrams that are exported as 
+     * Sets the default mime type for all diagrams that are exported as 
      * images.
      *
-     * @param mimetype  The MIME type to set as the default.
+     * @param mimeType  The MIME type to set as the default.
      */
-    void setImageMimetype(QString const & mimetype){m_imageMimetype=mimetype;};
+    void setImageMimeType(QString const & mimeType){m_imageMimeType=mimeType;};
 
     /**
-     * Gets the default mimetype for all diagrams that are exported as 
+     * Gets the default mime type for all diagrams that are exported as 
      * images.
      *
      * @return  The default MIME type for images.
      */
-    QString const & getImageMimetype()const{return m_imageMimetype;};
+    QString const & getImageMimeType()const{return m_imageMimeType;};
 
     /**
      * Carries out the cut/copy command with different action performed
@@ -300,10 +299,39 @@ public:
      */
     KTabWidget *tabWidget();
 
+    /**
+     * Returns the current text in the status bar.
+     *
+     * @return The text in the status bar.
+     */
+    QString getStatusBarMsg();
+
+    /**
+     * Returns the default code generation policy.
+     */
+    CodeGenerationPolicy *getCommonPolicy();
+
+    /**
+     * Sets the CodeGenPolicyExt object.
+     */
+    void setPolicyExt(CodeGenPolicyExt *policy);
+
+    /**
+     * Returns the CodeGenPolicyExt object.
+     */
+    CodeGenPolicyExt *getPolicyExt();
+
 protected:
     virtual void keyPressEvent(QKeyEvent* e);
     virtual void keyReleaseEvent(QKeyEvent* e);
 
+    /**
+     * Event handler to receive custom events.
+     * It handles events such as exporting all views from command line (in
+     * that case, it executes the exportAllViews method in the event).
+     */
+    virtual void customEvent(QCustomEvent* e);
+
     /**
      * Helper method for handling cursor key release events (refactoring).
      */
@@ -384,9 +412,7 @@ protected:
      */
     virtual void readProperties(KConfig *_cfg);
 
-    CodeGenerationPolicy * m_defaultcodegenerationpolicy;
-
-protected slots:
+    CodeGenerationPolicy * m_commoncodegenpolicy;
 
     /**
      * Updates the Menu for language selection and sets the
@@ -394,7 +420,9 @@ protected slots:
      * not one of the registered languages it tries to fall back
      * to Cpp
      */
-    void updateLangSelectMenu();
+    void updateLangSelectMenu(Uml::Programming_Language activeLanguage);
+
+protected slots:
 
     /**
      * Show "Tip of the Day" dialog
@@ -403,12 +431,10 @@ protected slots:
 
 public slots:
 
-
     /**
-     * Sets up generatorDict with all the available code
-     * generators and calls updateLangSelectMenu()
+     * Reads the activeLanguage from the KConfig and calls updateLangSelectMenu()
      */
-    void initGenerators();
+    void initGenerator();
 
     /**
      * Runs the code generation wizard.
@@ -460,7 +486,23 @@ public slots:
      * aborts.
      */
     void slotFileQuit();
-
+    
+     /**
+      * Exports the current model to docbook in a subdir of the 
+      * current model directory named from the model name.
+      * @todo Let the user chose the destination directory and 
+      * name, using network transparency.
+      */
+    void slotFileExportDocbook();
+    
+    /**
+     * Exports the current model to XHTML in a subdir of the 
+     * current model directory named from the model name.
+     * @todo Let the user chose the destination directory and 
+     * name, using network transparency.
+     */
+    void slotFileExportXhtml();
+    
     /**
      * Put the marked text/object into the clipboard and remove
      * it from the document.
@@ -596,17 +638,24 @@ public slots:
     /**
      * Get the language for import and code generation.
      */
-    QString getActiveLanguage() const;
+    Uml::Programming_Language getActiveLanguage();
 
     /**
      * Return true if the active language is case sensitive.
      */
-    bool activeLanguageIsCaseSensitive() const;
+    bool activeLanguageIsCaseSensitive();
 
     /**
      * Return the target language depedent scope separator.
      */
-    QString activeLanguageScopeSeparator() const;
+    QString activeLanguageScopeSeparator();
+
+    /**
+     * Return the default code generation language as configured by KConfig.
+     * If the activeLanguage is not found in the KConfig then use Uml::pl_Cpp
+     * as the default.
+     */
+    Uml::Programming_Language getDefaultLanguage();
 
     /**
      * Menu selection for clear current view.
@@ -628,6 +677,11 @@ public slots:
      */
     void slotCurrentViewExportImage();
 
+    /**
+     * Menu selection for exporting all views as images.
+     */
+    void slotAllViewsExportImage();
+
     /**
      * Menu selection for current view properties.
      */
@@ -748,6 +802,14 @@ public slots:
      */
     void slotMoveTabRight();
 
+    KConfig *getConfig() { return m_config; }
+
+    /**
+     * This slot deletes the current XHTML documentation generator as soon as
+     * this one signals that it has finished.
+     */
+    void slotXhtmlDocGenerationFinished();
+
 private:
     static UMLApp* s_instance;
 
@@ -764,14 +826,18 @@ private:
     /**
      * Active language.
      */
-    QString m_activeLanguage;
+    Uml::Programming_Language m_activeLanguage;
 
     /**
-     * Dictionary containing all the info needed to load a Code Generator
-     * at run time:
-     * Language (so that the user can choose), library and object name.
+     * Active code generator.
      */
-    GeneratorDict m_generatorDict;
+    CodeGenerator *m_codegen;
+
+    /**
+     * Active policy extension.
+     * Only used for new code generators ({Cpp,Java,Ruby}CodeGenerator)
+     */
+    CodeGenPolicyExt *m_policyext;
 
     /**
      *  Returns whether we can decode the given mimesource
@@ -792,7 +858,7 @@ private:
 
     /**
      * Initialize code generators at startup. 
-     * Why is this important? Because IF we dont do this, then changes
+     * Why is this important? Because IF we don't do this, then changes
      * to the UML may not be synced with the saved code generation params
      * for those languages which arent currently active.
      */
@@ -851,11 +917,13 @@ private:
     KAction* fileOpen;
     KRecentFilesAction* fileOpenRecent;
     KAction* fileSave;
-
     KAction* fileSaveAs;
     KAction* fileClose;
     KAction* filePrint;
     KAction* fileQuit;
+    KAction* fileExportDocbook;
+    KAction* fileExportXhtml;
+
     KAction* editCut;
     KAction* editCopy;
     KAction* editPaste;
@@ -879,6 +947,7 @@ private:
     KToggleAction* viewSnapToGrid;
     KToggleAction* viewShowGrid;
     KAction* viewExportImage;
+    KAction* viewExportImageAll;
     KAction* viewProperties;
 
     KAction* zoom100Action;
@@ -916,12 +985,6 @@ private:
      * True if the application is opening an existing document
      */
     bool m_loading;
-    Settings::OptionState m_optionState;
-
-    /**
-     * Blank widget, displayed when there are no diagrams
-     */
-    InfoWidget* blankWidget;
 
     /**
      * Shows, and is parent of, all the UMLViews (diagrams)
@@ -935,25 +998,26 @@ private:
      */
     KTabWidget* m_tabWidget;
 
-    ExportViewAction* viewExportAll;
-
     /**
-     * Default Mimetype to use for image export.
+     * Default mime type to use for image export.
      */
-    QString m_imageMimetype;
+    QString m_imageMimeType;
 
     /**
-     * the global UML settings dialogue
+     * the global UML settings dialog
      */
     SettingsDlg* m_dlg;
 
-public:
-    Settings::OptionState getOptionState() {
-        return m_optionState;
-    }
-    void setOptionState(Settings::OptionState optstate) {
-        m_optionState = optstate;
-    }
+    /**
+     * The UMLViewImageExporterAll used to export all the views.
+     */
+    UMLViewImageExporterAll* m_imageExporterAll;
+
+    /**
+     * The running XHTML documentation generator. null when no generation is
+     * running
+     */
+    XhtmlGenerator* m_xhtmlGenerator;
 
 signals:
 
