@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2002-2005
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,36 +5,46 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *  copyright (C) 2002-2006                                                *
+ *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
  ***************************************************************************/
 
+// own header
+#include "umlobject.h"
+// qt/kde includes
 #include <qregexp.h>
 #include <kdebug.h>
 #include <kapplication.h>
-#include "umlobject.h"
+// app includes
+#include "uniqueid.h"
 #include "uml.h"
 #include "umldoc.h"
 #include "umllistview.h"
 #include "umllistviewitem.h"
 #include "package.h"
+#include "folder.h"
 #include "stereotype.h"
+#include "object_factory.h"
 #include "model_utils.h"
-#include "import_utils.h"
+#include "codeimport/import_utils.h"
+#include "docwindow.h"
+#include "dialogs/classpropdlg.h"
 
 UMLObject::UMLObject(const UMLObject * parent, const QString &name, Uml::IDType id)
         : QObject(const_cast<UMLObject*>(parent), "UMLObject" ) {
     init();
     if (id == Uml::id_None)
-        m_nId = UMLApp::app()->getDocument()->getUniqueID();
+        m_nId = UniqueID::gen();
     else
         m_nId = id;
     m_Name = name;
 }
 
 UMLObject::UMLObject(const QString &name, Uml::IDType id)
-        :  QObject(UMLApp::app()->getDocument(), "UMLObject") {
+        :  QObject(UMLApp::app()->getDocument()) {
     init();
     if (id == Uml::id_None)
-        m_nId = UMLApp::app()->getDocument()->getUniqueID();
+        m_nId = UniqueID::gen();
     else
         m_nId = id;
     m_Name = name;
@@ -68,34 +73,18 @@ void UMLObject::init() {
     m_pSecondary = NULL;
 }
 
-
-bool UMLObject::isSavedInSeparateFile() {
-    if (UMLApp::app()->getOptionState().generalState.tabdiagrams) {
-        // Umbrello currently does not support external folders
-        // when tabbed diagrams are enabled.
-        return false;
+bool UMLObject::showProperties(int page, bool assoc) {
+    DocWindow *docwindow = UMLApp::app()->getDocWindow();
+    docwindow->updateDocumentation(false);
+    ClassPropDlg* dlg = new ClassPropDlg((QWidget*)UMLApp::app(), this, page, assoc);
+    bool modified = false;
+    if (dlg->exec()) {
+        docwindow->showDocumentation(this, true);
+        UMLApp::app()->getDocument()->setModified(true);
+        modified = true;
     }
-    const QString msgPrefix("UMLObject::isSavedInSeparateFile(" + m_Name + "): ");
-    UMLListView *listView = UMLApp::app()->getListView();
-    UMLListViewItem *lvItem = listView->findUMLObject(this);
-    if (lvItem == NULL) {
-        kdError() << msgPrefix
-        << "listView->findUMLObject(this) returns false"
-        << endl;
-        return false;
-    }
-    UMLListViewItem *parentItem = dynamic_cast<UMLListViewItem*>( lvItem->parent() );
-    if (parentItem == NULL) {
-        kdError() << msgPrefix
-        << "parent item in listview is not a UMLListViewItem (?)"
-        << endl;
-        return false;
-    }
-    const Uml::ListView_Type lvt = parentItem->getType();
-    if (! UMLListView::typeIsFolder(lvt))
-        return false;
-    QString folderFile = parentItem->getFolderFile();
-    return !folderFile.isEmpty();
+    dlg->close(true); //wipe from memory
+    return modified;
 }
 
 bool UMLObject::acceptAssociationType(Uml::Association_Type)
@@ -117,13 +106,31 @@ QString UMLObject::getName() const {
     return m_Name;
 }
 
-QString UMLObject::getFullyQualifiedName(QString separator) const {
+QString UMLObject::getFullyQualifiedName(QString separator,
+                                         bool includeRoot /* = false */) const {
     QString fqn;
     if (m_pUMLPackage) {
-        if (separator.isEmpty())
-            separator = UMLApp::app()->activeLanguageScopeSeparator();
-        fqn = m_pUMLPackage->getFullyQualifiedName(separator);
-        fqn.append(separator);
+        bool skipPackage = false;
+        if (!includeRoot) {
+            UMLDoc *umldoc = UMLApp::app()->getDocument();
+            if (m_pUMLPackage == umldoc->getDatatypeFolder()) {
+                skipPackage = true;
+            } else {
+                for (int i = 0; i < Uml::N_MODELTYPES; i++) {
+                    const Uml::Model_Type mt = (Uml::Model_Type)i;
+                    if (m_pUMLPackage == umldoc->getRootFolder(mt)) {
+                        skipPackage = true;
+                        break;
+                    }
+                }
+            }
+        }
+        if (!skipPackage) {
+            if (separator.isEmpty())
+                separator = UMLApp::app()->activeLanguageScopeSeparator();
+            fqn = m_pUMLPackage->getFullyQualifiedName(separator, includeRoot);
+            fqn.append(separator);
+        }
     }
     fqn.append(m_Name);
     return fqn;
@@ -185,8 +192,6 @@ bool UMLObject::operator==(UMLObject & rhs ) {
 
 void UMLObject::copyInto(UMLObject *rhs) const
 {
-    UMLDoc *umldoc = UMLApp::app()->getDocument();
-
     // Data members with copy constructor
     rhs->m_Doc = m_Doc;
     rhs->m_pStereotype = m_pStereotype;
@@ -197,10 +202,10 @@ void UMLObject::copyInto(UMLObject *rhs) const
     rhs->m_pUMLPackage = m_pUMLPackage;
 
     // We don't want the same name existing twice.
-    rhs->m_Name = umldoc->uniqObjectName(m_BaseType, m_Name, m_pUMLPackage);
+    rhs->m_Name = Model_Utils::uniqObjectName(m_BaseType, m_pUMLPackage, m_Name);
 
     // Create a new ID.
-    rhs->m_nId = umldoc->getUniqueID();
+    rhs->m_nId = UniqueID::gen();
 
     // Hope that the parent from QObject is okay.
     if (rhs->parent() != parent())
@@ -247,6 +252,10 @@ Uml::Object_Type UMLObject::getBaseType() const {
     return m_BaseType;
 }
 
+void UMLObject::setBaseType(Uml::Object_Type ot) {
+    m_BaseType = ot;
+}
+
 Uml::IDType UMLObject::getID() const {
     return m_nId;
 }
@@ -279,6 +288,7 @@ void UMLObject::setUMLStereotype(UMLStereotype *stereo) {
         }
     }
     m_pStereotype = stereo;
+    // TODO: don't emit modified() if predefined folder
     emit modified();
 }
 
@@ -296,16 +306,20 @@ void UMLObject::setPackage(const QString &_name) {
     UMLObject *pkgObj = NULL;
     if (!_name.isEmpty()) {
         UMLDoc* umldoc = UMLApp::app()->getDocument();
-        if (umldoc == NULL) {
-            kdError() << "UMLObject::setPackage: cannot set package name on "
-            << m_Name << endl;
-            return;
-        }
-        pkgObj = umldoc->findUMLObject(_name, Uml::ot_Package);
+        pkgObj = umldoc->findUMLObject(_name);
         if (pkgObj == NULL) {
             kdDebug() << "UMLObject::setPackage: creating UMLPackage "
-            << _name << " for " << m_Name << endl;
+                << _name << " for " << m_Name << endl;
             pkgObj = Import_Utils::createUMLObject(Uml::ot_Package, _name);
+        } else {
+            const Uml::Object_Type ot = pkgObj->getBaseType();
+            if (ot != Uml::ot_Package && ot != Uml::ot_Folder && ot != Uml::ot_Component) {
+                kdError() << "UMLObject::setPackage(" << m_Name << "): "
+                    << "existing " << _name << " is not a container" << endl;
+                // This should not happen - if it does, there may be further problems.
+                // A container name should not overlap with another name in the same scope.
+                pkgObj = Import_Utils::createUMLObject(Uml::ot_Package, _name);
+            }
         }
     }
     setUMLPackage( static_cast<UMLPackage *>(pkgObj) );
@@ -320,36 +334,32 @@ const UMLStereotype * UMLObject::getUMLStereotype() {
     return m_pStereotype;
 }
 
-QString UMLObject::getStereotype(bool includeAdornments /* = false */) {
+QString UMLObject::getStereotype(bool includeAdornments /* = false */) const {
     if (m_pStereotype == NULL)
         return "";
     QString name = m_pStereotype->getName();
     if (includeAdornments)
-        name = "" + name + "";
+        name = QString::fromUtf8("«") + name + QString::fromUtf8("»");
     return name;
 }
 
-QString UMLObject::getPackage(QString separator) {
-    if (m_pUMLPackage == NULL)
+QString UMLObject::getPackage(QString separator, bool includeRoot) {
+    QString fqn = getFullyQualifiedName(separator, includeRoot);
+    if (!fqn.contains(separator))
         return "";
-    QStringList pkgList;
-    UMLPackage* pkg = m_pUMLPackage;
-    while (pkg != NULL) {
-        pkgList.prepend(pkg->getName());
-        pkg = pkg->getUMLPackage();
-    }
-    if (separator.isEmpty())
-        separator = UMLApp::app()->activeLanguageScopeSeparator();
-    return pkgList.join(separator);
+    QString scope = fqn.left(fqn.length() - separator.length() - m_Name.length());
+    return scope;
 }
 
-UMLPackageList UMLObject::getPackages() const {
+UMLPackageList UMLObject::getPackages(bool includeRoot) const {
     UMLPackageList pkgList;
     UMLPackage* pkg = m_pUMLPackage;
     while (pkg != NULL) {
         pkgList.prepend(pkg);
         pkg = pkg->getUMLPackage();
     }
+    if (!includeRoot)
+        pkgList.removeFirst();
     return pkgList;
 }
 
@@ -361,6 +371,18 @@ QString UMLObject::getSecondaryId() const {
     return m_SecondaryId;
 }
 
+void UMLObject::setSecondaryId(QString id) {
+    m_SecondaryId = id;
+}
+
+QString UMLObject::getSecondaryFallback() const {
+    return m_SecondaryFallback;
+}
+
+void UMLObject::setSecondaryFallback(QString id) {
+    m_SecondaryFallback = id;
+}
+
 void UMLObject::maybeSignalObjectCreated() {
     if (!m_bCreationWasSignalled &&
             m_BaseType != Uml::ot_Stereotype &&
@@ -373,29 +395,44 @@ void UMLObject::maybeSignalObjectCreated() {
 }
 
 bool UMLObject::resolveRef() {
-    if (m_pSecondary || m_SecondaryId.isEmpty()) {
+    if (m_pSecondary || (m_SecondaryId.isEmpty() && m_SecondaryFallback.isEmpty())) {
         maybeSignalObjectCreated();
         return true;
     }
 #ifdef VERBOSE_DEBUGGING
     kdDebug() << "UMLObject::resolveRef(" << m_Name << "): m_SecondaryId is "
-    << m_SecondaryId << endl;
+              << m_SecondaryId << endl;
 #endif
     UMLDoc *pDoc = UMLApp::app()->getDocument();
     // In the new, XMI standard compliant save format,
     // the type is the xmi.id of a UMLClassifier.
-    Uml::IDType id = STR2ID(m_SecondaryId);
-    m_pSecondary = pDoc->findObjectById(id);
-    if (m_pSecondary != NULL) {
-        m_SecondaryId = "";
-        maybeSignalObjectCreated();
-        return true;
+    if (! m_SecondaryId.isEmpty()) {
+        m_pSecondary = pDoc->findObjectById(STR2ID(m_SecondaryId));
+        if (m_pSecondary != NULL) {
+            if (m_pSecondary->getBaseType() == Uml::ot_Stereotype) {
+                m_pStereotype = static_cast<UMLStereotype*>(m_pSecondary);
+                m_pStereotype->incrRefCount();
+                m_pSecondary = NULL;
+            }
+            m_SecondaryId = "";
+            maybeSignalObjectCreated();
+            return true;
+        }
     }
     if (!pDoc->isNativeXMIFile() || !m_SecondaryId.contains(QRegExp("\\D"))) {
-        kdError() << "UMLObject::resolveRef(" << m_Name
-        << "): cannot find type with id "
-        << ID2STR(id) << endl;
-        return false;
+        if (m_SecondaryFallback.isEmpty()) {
+            kdError() << "UMLObject::resolveRef(" << m_Name
+                      << "): cannot find type with id "
+                      << m_SecondaryId << endl;
+            return false;
+        }
+#ifdef VERBOSE_DEBUGGING
+        kdDebug() << "UMLObject::resolveRef(" << m_Name 
+                  << "): could not resolve secondary ID " << m_SecondaryId
+                  << ", using secondary fallback " << m_SecondaryFallback
+                  << endl;
+#endif
+        m_SecondaryId = m_SecondaryFallback;
     }
     // Assume we're dealing with the older Umbrello format where
     // the type name was saved in the "type" attribute rather
@@ -407,10 +444,10 @@ bool UMLObject::resolveRef() {
         return true;
     }
     //pDoc->setIsOldFile(true);
-    // Work around UMLDoc::createUMLObject()'s incapability
+    // Work around Object_Factory::createUMLObject()'s incapability
     // of on-the-fly scope creation:
     if (m_SecondaryId.contains("::")) {
-        // TODO: Merge Import_Utils::createUMLObject() into UMLDoc::createUMLObject()
+        // TODO: Merge Import_Utils::createUMLObject() into Object_Factory::createUMLObject()
         m_pSecondary = Import_Utils::createUMLObject(Uml::ot_UMLObject, m_SecondaryId, NULL);
         if (m_pSecondary) {
             if (Import_Utils::newUMLObjectWasCreated()) {
@@ -443,7 +480,7 @@ bool UMLObject::resolveRef() {
         if (Model_Utils::isCommonDataType(m_SecondaryId))
             ot = Uml::ot_Datatype;
     }
-    m_pSecondary = pDoc->createUMLObject(ot, m_SecondaryId, NULL);
+    m_pSecondary = Object_Factory::createUMLObject(ot, m_SecondaryId, NULL);
     if (m_pSecondary == NULL)
         return false;
     m_SecondaryId = "";
@@ -471,6 +508,16 @@ QDomElement UMLObject::save( const QString &tag, QDomDocument & qDoc ) {
     }
     qElement.setAttribute( "xmi.id", ID2STR(m_nId) );
     qElement.setAttribute( "name", m_Name );
+    if (m_BaseType != Uml::ot_Operation &&
+            m_BaseType != Uml::ot_Role &&
+            m_BaseType != Uml::ot_Attribute) {
+        Uml::IDType nmSpc;
+        if (m_pUMLPackage)
+            nmSpc = m_pUMLPackage->getID();
+        else
+            nmSpc = UMLApp::app()->getDocument()->getModelID();
+        qElement.setAttribute( "namespace", ID2STR(nmSpc) );
+    }
     if (! m_Doc.isEmpty())
         qElement.setAttribute( "comment", m_Doc );  //CHECK: uml13.dtd compliance
 #ifdef XMI_FLAT_PACKAGES
@@ -509,7 +556,7 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
         if (m_BaseType == Uml::ot_Role) {
             // Before version 1.4, Umbrello did not save the xmi.id
             // of UMLRole objects.
-            m_nId = umldoc->getUniqueID();
+            m_nId = UniqueID::gen();
         } else {
             kdError() << "UMLObject::loadFromXMI(" << m_Name
             << "): nonexistent or illegal xmi.id" << endl;
@@ -616,9 +663,9 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
     }
     /**** End of XMI_FLAT_PACKAGES and old files handling ****************/
 
-    // If the name is not set, let's check whether the attributes are saved
-    // as child nodes.
-    if (m_Name.isEmpty()) {
+    // If the node has child nodes, check whether attributes can be
+    // extracted from them.
+    if (element.hasChildNodes()) {
         QDomNode node = element.firstChild();
         if (node.isComment())
             node = node.nextSibling();
@@ -649,6 +696,29 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
                 if (ownerScope.isEmpty())
                     ownerScope = elem.text();
                 m_bStatic = (ownerScope == "classifier");
+            } else if (Uml::tagEq(tag, "stereotype")) {
+                QString stereo = elem.attribute("xmi.value", "");
+                if (stereo.isEmpty() && elem.hasChildNodes()) {
+                    /* like so:
+                     <UML:ModelElement.stereotype>
+                       <UML:Stereotype xmi.idref = '07CD'/>
+                     </UML:ModelElement.stereotype>
+                     */
+                    QDomNode stereoNode = elem.firstChild();
+                    QDomElement stereoElem = stereoNode.toElement();
+                    tag = stereoElem.tagName();
+                    if (Uml::tagEq(tag, "Stereotype")) {
+                        stereo = stereoElem.attribute("xmi.idref", "");
+                    }
+                }
+                if (! stereo.isEmpty()) {
+                    Uml::IDType stereoID = STR2ID(stereo);
+                    m_pStereotype = umldoc->findStereotypeById(stereoID);
+                    if (m_pStereotype)
+                        m_pStereotype->incrRefCount();
+                    else
+                        m_SecondaryId = stereo;  // leave it to resolveRef()
+                }
             }
             node = node.nextSibling();
             if (node.isComment())
@@ -669,20 +739,19 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
             UMLListViewItem *parentItem = (UMLListViewItem*)listView->currentItem();
             if (parentItem) {
                 Uml::ListView_Type lvt = parentItem->getType();
-                if (lvt == Uml::lvt_Package ||
+                if (Model_Utils::typeIsContainer(lvt) ||
                         lvt == Uml::lvt_Class ||
                         lvt == Uml::lvt_Interface) {
                     UMLObject *o = parentItem->getUMLObject();
                     m_pUMLPackage = static_cast<UMLPackage*>( o );
                 }
             }
-        }
-
-        if (m_pUMLPackage)
+        } else if (m_pUMLPackage) {
             m_pUMLPackage->addObject(this);
-        else
-            umldoc->addUMLObject(this);
-        //kapp->processEvents();
+        } else if (umldoc->rootFolderType(this) == Uml::N_MODELTYPES) {
+            kdDebug() << "UMLObject::loadFromXMI(" << m_Name << "): m_pUMLPackage is not set"
+                << endl;
+        }
     }
     return load(element);
 }
