@@ -5,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2002-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                 *
  ***************************************************************************/
 
 #ifndef UMLVIEW_H
@@ -31,13 +33,14 @@
 class ClassOptionsPage;
 class IDChangeLog;
 class ListPopupMenu;
-class FloatingText;
+class FloatingTextWidget;
 class SeqLineWidget;
 class ObjectWidget;
-
+class UMLFolder;
 class UMLApp;
 class UMLDoc;
 class UMLCanvasObject;
+class UMLViewImageExporter;
 
 class KPrinter;
 class ToolBarState;
@@ -56,22 +59,34 @@ class ToolBarStateFactory;
 class UMLView : public QCanvasView {
     Q_OBJECT
 public:
-    friend class ExportViewAction;
-    static QString imageTypeToMimeType(QString imagetype);
-    static QString mimeTypeToImageType(QString mimetype);
-    
-    /**
-     * Constructor for the main view
-     */
-    UMLView();
+    friend class UMLViewImageExporterModel;
 
     /**
-     * Destructor for the main view
+     * Constructor
      */
-    ~UMLView();
+    UMLView(UMLFolder *parentFolder);
+
+    /**
+     * Destructor
+     */
+    virtual ~UMLView();
 
     // Accessors and other methods dealing with loaded/saved data
 
+    /**
+     * Return the UMLFolder in which this diagram lives.
+     */
+    UMLFolder *getFolder() {
+        return m_pFolder;
+    }
+
+    /**
+     * Set the UMLFolder in which this diagram lives.
+     */
+    void setFolder(UMLFolder *folder) {
+        m_pFolder = folder;
+    }
+
     /**
      * Return the documentation of the diagram.
      */
@@ -336,20 +351,6 @@ public:
      */
     void print(KPrinter *pPrinter, QPainter & pPainter);
 
-    /**
-     * Print the entire diagram in this view to the file 'filename'
-     * @param isEPS  The file is an eps file and needs adjusting
-     *   of the eps bounding box values.
-     */
-    void printToFile(const QString &filename, bool isEPS);
-
-    /**
-     * Fix the file 'filename' to be a valid EPS containing the
-     * specified area (rect) of the diagram.
-     * Corrects the bounding box.
-     */
-    void fixEPS(const QString &filename, QRect rect);
-
     /**
      * Overrides the standard operation.
      */
@@ -360,14 +361,6 @@ public:
      */
     void showEvent(QShowEvent *se);
 
-    /**
-     * Sets an association to include the given widget.  If this is the second
-     * widget set for the association it creates the association.
-     *
-     * @param w The widget to set for the association.
-     */
-    bool setAssoc(UMLWidget *w);
-
     /**
      * Sees if a message is relevant to the given widget.  If it does delete it.
      * @param w The widget to check messages against.
@@ -403,6 +396,13 @@ public:
      */
     AssociationWidget * findAssocWidget(Uml::Association_Type at,
                                         UMLWidget *pWidgetA, UMLWidget *pWidgetB);
+
+    /**
+     * calls findAssocWidget on three possible types:
+     * Uml::at_Aggregation << Uml::at_Composition << Uml::at_Containment
+     */
+    AssociationWidget * findAssocWidget(UMLWidget *pWidgetA, UMLWidget *pWidgetB);
+
     /**
      * Remove a widget from view.
      *
@@ -423,15 +423,6 @@ public:
      */
     void clearSelected();
 
-    /**
-     * Move all the selected widgets.
-     *
-     * @param w The widget in the selected list to move in reference to.
-     * @param x The distance to move horizontally.
-     * @param y The distance to move vertically.
-     */
-    void moveSelected(UMLWidget * w, int x, int y);
-
     /**
      * Move all the selected widgets by a relative X and Y offset.
      *
@@ -443,11 +434,12 @@ public:
     /**
      * Return the amount of widgets selected.
      *
-     * @return Return the amount of widgets selected.
+     * @param filterText  When true, do NOT count floating text widgets that
+     *                    belong to other widgets (i.e. only count tr_Floating.)
+     *                    Default: Count all widgets.
+     * @return  Number of widgets selected.
      */
-    int getSelectCount() const {
-        return m_SelectedList.count();
-    }
+    int getSelectCount(bool filterText = false) const;
 
     /**
      * Set the useFillColor variable to all selected widgets
@@ -540,7 +532,6 @@ public:
      */
     void resetToolbar() {
         emit sigResetToolBar();
-        m_CurrentCursor = WorkToolBar::tbb_Arrow;
     }
 
     /**
@@ -576,8 +567,13 @@ public:
 
     /**
      * Fills the List with all the selected widgets from the diagram
+     * The list can be filled with all the selected widgets, or be filtered to prevent
+     * text widgets other than tr_Floating to be append.
+     *
+     * @param WidgetList The UMLWidgetList to fill.
+     * @param filterText Don't append the text, unless their role is tr_Floating
      */
-    bool getSelectedWidgets(UMLWidgetList& WidgetList);
+    bool getSelectedWidgets(UMLWidgetList& WidgetList, bool filterText = true);
 
     /**
      * Activate the view after a load a new file
@@ -659,12 +655,12 @@ public:
      */
     void copyAsImage(QPixmap*& pix);
 
-   
     /**
-     * Saves a png file to the given url.
-     @todo - rewrite this in terms of ExportViewAction? 
+     * Returns the imageExporter used to export the view.
+     *
+     * @return The imageExporter used to export the view.
      */
-    void exportImage();
+    UMLViewImageExporter* getImageExporter();
 
     /**
      * Adds an association to the view from the given data.
@@ -787,25 +783,25 @@ public:
 
     /**
      * Asks for confirmation and clears everything on the diagram.
-     * Called from menues.
+     * Called from menus.
      */
     void clearDiagram();
 
     /**
      * Changes snap to grid boolean.
-     * Called from menues.
+     * Called from menus.
      */
     void toggleSnapToGrid();
 
     /**
      * Changes snap to grid for component size boolean.
-     * Called from menues.
+     * Called from menus.
      */
     void toggleSnapComponentSizeToGrid();
 
     /**
      *  Changes show grid boolean.
-     * Called from menues.
+     * Called from menus.
      */
     void toggleShowGrid();
 
@@ -872,49 +868,10 @@ public:
     ObjectWidget * onWidgetLine( const QPoint &point );
 
     /**
-     * Return the line used to show a join between objects as an association
-     * is being made.
+     * Return pointer to the first selected widget (for multi-selection)
      */
-    QCanvasLine * getAssocLine() {
-        return m_pAssocLine;
-    }
-
-    /**
-     * Set the line used to show a join between objects as an association
-     * is being made.
-     */
-    void setAssocLine(QCanvasLine * line) {
-        m_pAssocLine = line;
-    }
-
-    /**
-     * Return pointer to the first selected widget when creating an
-     * association.
-     */
-    UMLWidget* getFirstSelectedWidget() {
-        return m_pFirstSelectedWidget;
-    }
-
-    /**
-     * Set pointer to the first selected widget when creating an
-     * association.
-     */
-    void setFirstSelectedWidget(UMLWidget* w) {
-        m_pFirstSelectedWidget = w;
-    }
-
-    /**
-     * Return the current UMLWidget we are on.
-     */
-    UMLWidget * getOnWidget() {
-        return m_pOnWidget;
-    }
-
-    /**
-     * Set the current UMLWidget we are on.
-     */
-    void setOnWidget(UMLWidget * w) {
-        m_pOnWidget = w;
+    UMLWidget* getFirstMultiSelectedWidget() {
+        return m_SelectedList.first();
     }
 
     /**
@@ -946,20 +903,6 @@ public:
         m_bCreateObject = bCreate;
     }
 
-    /**
-     * Return the current AssociationWidget we are moving.
-     */
-    AssociationWidget * getMoveAssoc() {
-        return m_pMoveAssoc;
-    }
-
-    /**
-     * Set the current AssociationWidget we are moving.
-     */
-    void setMoveAssoc(AssociationWidget *aw) {
-        m_pMoveAssoc = aw;
-    }
-
     /**
      * Emit the sigRemovePopupMenu Qt signal.
      */
@@ -1094,12 +1037,6 @@ protected:
     void contentsDropEvent(QDropEvent* mouseEvent);
 
 
-    /**
-     * Adds an AssociationWidget to the association list
-     * and creates the corresponding UMLAssociation in the current UMLDoc.
-     */
-    void addAssocInViewAndDoc(AssociationWidget* assoc);
-
     /**
      * Gets the smallest area to print.
      *
@@ -1107,7 +1044,7 @@ protected:
      */
     QRect getDiagramRect();
 
-    
+
     /**
      * Initializes key variables.
      */
@@ -1115,21 +1052,25 @@ protected:
 
     /**
      * Overrides the standard operation.
+     * Calls the same method in the current tool bar state.
      */
     void contentsMouseReleaseEvent(QMouseEvent* mouseEvent);
 
     /**
      * Overrides the standard operation.
+     * Calls the same method in the current tool bar state.
      */
     void contentsMouseMoveEvent(QMouseEvent* mouseEvent);
 
     /**
      * Override standard method.
+     * Calls the same method in the current tool bar state.
      */
     void contentsMouseDoubleClickEvent(QMouseEvent* mouseEvent);
 
     /**
      * Override standard method.
+     * Calls the same method in the current tool bar state.
      */
     void contentsMousePressEvent(QMouseEvent* mouseEvent);
 
@@ -1152,10 +1093,10 @@ protected:
     void updateComponentSizes();
 
     /**
-     * Find the maximum bounding rectangle of FloatingText widgets.
+     * Find the maximum bounding rectangle of FloatingTextWidget widgets.
      * Auxiliary to copyAsImage().
      *
-     * @param ft Pointer to the FloatingText widget to consider.
+     * @param ft Pointer to the FloatingTextWidget widget to consider.
      * @param px  X coordinate of lower left corner. This value will be
      *            updated if the X coordinate of the lower left corner
      *            of ft is smaller than the px value passed in.
@@ -1169,7 +1110,7 @@ protected:
      *            updated if the Y coordinate of the upper right corner
      *            of ft is larger than the qy value passed in.
      */
-    void findMaxBoundingRectangle(const FloatingText* ft,
+    void findMaxBoundingRectangle(const FloatingTextWidget* ft,
                                   int& px, int& py, int& qx, int& qy);
 
     void forceUpdateWidgetFontMetrics(QPainter *painter);
@@ -1180,31 +1121,31 @@ protected:
     int m_nCollaborationId;
 
     QPoint m_Pos;
-    bool m_bCreateObject, m_bDrawRect, m_bDrawSelectedOnly, m_bPaste;
+    bool m_bCreateObject, m_bDrawSelectedOnly, m_bPaste;
     ListPopupMenu * m_pMenu;
     UMLWidgetList m_SelectedList;
-    AssociationWidget * m_pMoveAssoc;
 
     /**
      *  Flag if view/children started cut operation.
      */
     bool m_bStartedCut;
 
-public:
-    WorkToolBar::ToolBar_Buttons getCurrentCursor() const;
-
 private:
+    /**
+     * The folder in which this UMLView is contained
+     */
+    UMLFolder *m_pFolder;
+
+    /**
+     * set to true when a child has used the showDocumentation method,
+     * thus when one clicks on a child widget.
+     * Reset to false when clicking in an empty region of the view.
+     */
+    bool m_bChildDisplayedDoc;
 
     ToolBarStateFactory* m_pToolBarStateFactory;
     ToolBarState* m_pToolBarState;
 
-    WorkToolBar::ToolBar_Buttons m_CurrentCursor;
-
-    /**
-     * converts toolbar button enums to association type enums
-     */
-    Uml::Association_Type convert_TBB_AT(WorkToolBar::ToolBar_Buttons tbb);
-
     /**
      * LocalID Changes Log for paste actions
      */
@@ -1216,28 +1157,12 @@ private:
      */
     bool m_bActivated;
 
-    /**
-     * It holds a pointer the the first selected widget when creating an Association
-     */
-    UMLWidget* m_pFirstSelectedWidget;
-
     /**
      * Status of a popupmenu on view.
      * true - a popup is on view
      */
     bool m_bPopupShowing;
 
-    /**
-     * The current @ref UMLWidget we are on. A reference is
-     * kept so we can monitor for a leave event.
-     */
-    UMLWidget * m_pOnWidget;
-
-    /**
-     * The Line used to show a join between objects as an association is being made.
-     */
-    QCanvasLine * m_pAssocLine;
-
     /**
      * The offset at which to paste the clipboard.
      */
@@ -1254,20 +1179,19 @@ private:
     UMLDoc* m_pDoc;
 
     /**
-     * The url of the last saved image
+     * The UMLViewImageExporter used to export the view.
      */
-    KURL m_ImageURL;
-
-    /**
-     * Used by @ref contentsMouseMoveEvent() to know if a mouse button is pressed.
-     */
-    bool m_bMouseButtonPressed;
+    UMLViewImageExporter* m_pImageExporter;
 
 public slots:
 
     void zoomIn();
     void zoomOut();
 
+    /**
+     * Changes the current tool to the selected tool.
+     * The current tool is cleaned and the selected tool initialized.
+     */
     void slotToolBarChanged(int c);
     void slotObjectCreated(UMLObject * o);
     void slotObjectRemoved(UMLObject * o);
@@ -1282,7 +1206,7 @@ public slots:
      * This slot is entered when an event has occurred on the views display,
      * most likely a mouse event.  Before it sends out that mouse event everyone
      * that displays a menu on the views surface (widgets and this ) thould remove any
-     * menu.  This stops more then one menu bieing diplayed.
+     * menu.  This stops more then one menu bieing displayed.
      */
     void slotRemovePopupMenu();
 
@@ -1320,6 +1244,16 @@ signals:
     void sigSnapToGridToggled(bool);
     void sigSnapComponentSizeToGridToggled(bool);
     void sigShowGridToggled(bool);
+
+    /**
+     * Emitted when an association is removed.
+     */
+    void sigAssociationRemoved(AssociationWidget*);
+
+    /**
+     * Emitted when a widget is removed.
+     */
+    void sigWidgetRemoved(UMLWidget*);
 };
 
 #endif // UMLVIEW_H
